//////////////////////////////////////////////////////////////////////////////
//                                                                         
//  NAME: Daniel Abadi
//  FILE: VC.C
//  PROJ: COKO Debugger (Visual COKO)
//  DATE: SUMMER 2001
//
// SYNOPSIS
//  Methods definitions for MyMainWindow (Visual COKO) class.
//  
//////////////////////////////////////////////////////////////////////////////

#include "VC.moc"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: MyMainWindow
// Parameters:	(DDebugController* db, RuleBlockList* c, RuleBlockList* b,
//               DTransformationList* transList)
//
// Constructor.
//
//////////////////////////////////////////////////////////////////////////////

MyMainWindow::MyMainWindow(DDebugController* db, RuleBlockList* c, RuleBlockList* b, DTransformationList* transList):QMainWindow(NULL, "Visual COKO")
{

  //set window size
  setGeometry(100,100,400,400);

  //initialize file menu
  file = new QPopupMenu(this);
  file->insertSeparator();
  file->insertItem("Exit", this, SLOT(quit()));

  //initialize commands menu
  commands = new QPopupMenu(this);
  commandsIndicies[0] = commands->insertItem("Step", this, SLOT(step()));
  commandsIndicies[1] = commands->insertItem("Next", this, SLOT(next()));
  commandsIndicies[2] = commands->insertItem("Continue", this, SLOT(cont()));
  commandsIndicies[3] = commands->insertItem("Break", this, SLOT(doBreak()));
  commandsIndicies[4] = commands->insertItem("Condition", this, SLOT(doCondition()));
  commandsIndicies[5] = commands->insertItem("Clear", this, SLOT(doClear()));
  commandsIndicies[6] = commands->insertItem("Run", this, SLOT(doRun()));
  commandsIndicies[7] = commands->insertItem("Stack", this, SLOT(doStack()));

  //initialize view menu
  view = new QPopupMenu(this);
  view->setCheckable(true);
  viewIndicies[0] = view->insertItem("Root KOLA Tree", this, SLOT(showRT()));
  viewIndicies[1] = view->insertItem("Current KOLA Tree", this, SLOT(showCT()));
  viewIndicies[2] = view->insertItem("Stack", this, SLOT(toggleStack()));
  view->setItemChecked(viewIndicies[0], true);
  view->setItemChecked(viewIndicies[1], false);
  view->setItemChecked(viewIndicies[1], false);

  //intialize help menu
  help = new QPopupMenu(this);
  help->insertItem("Contents");
  help->insertSeparator();
  help->insertItem("About");

  //intialize menu bar
  menuBar()->insertItem("File", file);
  menuBar()->insertItem("Commands", commands);
  menuBar()->insertItem("View", view);
  menuBar()->insertItem("Help", help);

  blankicon = QPixmap();
  //nexticon = QPixmap(breakat_xpm);


  //initialize tool bar (below menu bar)
  toolbar = new QToolBar(this);

  //initialize step button
  b1 = new QToolButton(blankicon, "Step", "Step to the next COKO statement.", this, SLOT(step()), toolbar);
  QFont b_font(  b1->font() );
  b_font.setBold( TRUE );
  b1->setFont( b_font ); 
  b1->setText( tr( "Step" ) );

  //initialize next button
  b2 = new QToolButton(blankicon, "Next", "Step to the next COKO statement in this transformation.", this, SLOT(next()), toolbar);
  b2->setFont( b_font ); 
  b2->setText( tr( "Next" ) );

  //initialize continue button
  b3 = new QToolButton(blankicon, "Continue", "Continue executing until some kind of break is encountered.", this, SLOT(cont()), toolbar);
  b3->setFont( b_font ); 
  b3->setText( tr( "Cont" ) );

  //initialize break button
  b4 = new QToolButton(blankicon, "Break", "Break at marked statement.", this, SLOT(doBreak()), toolbar);
  b4->setFont( b_font ); 
  b4->setText( tr( "Break" ) );

  //initialize condition button
  b5 = new QToolButton(blankicon, "Condition", "Place a conditional break at marked statement.", this, SLOT(doCondition()), toolbar);
  b5->setFont( b_font ); 
  b5->setText( tr( "Cond" ) );

  //initialize clear button
  b6 = new QToolButton(blankicon, "Clear", "Clear all breaks at marked statement.", this, SLOT(doClear()), toolbar);
  b6->setFont( b_font ); 
  b6->setText( tr( "Clear" ) );

  //initialize run button
  b7 = new QToolButton(blankicon, "Run", "Restarts the debugger.", this, SLOT(doRun()), toolbar);
  b7->setFont( b_font ); 
  b7->setText( tr( "Run" ) );

  //initialize stack button
  b8 = new QToolButton(blankicon, "Stack", "Shows the current stack of transformations.", this, SLOT(doStack()), toolbar);
  b8->setFont( b_font ); 
  b8->setText( tr( "Stack" ) );

  //initialize exit button
  b9 = new QToolButton(blankicon, "Exit", "Exit Visual COKO.", this, SLOT(quit()), toolbar);
  b9->setFont( b_font ); 
  b9->setText( tr( "Exit" ) );

  //initialize status bar
  statusBar();

  //initialize root tree display window (rtGB) and current tree display window (ctGB)
  rtGB = new DScrollWidget(view, viewIndicies[0], NULL, NULL, NULL);
  ctGB = new DScrollWidget(view, viewIndicies[1], NULL, NULL, NULL);
  rtWindow = new DPaintWindow(view, viewIndicies[0], this, rtGB, "Root  KOLA Tree Display");
  ctWindow = new DPaintWindow(view, viewIndicies[1], this, ctGB, "Curent KOLA Tree Display");
  rtGB->setSon(rtWindow);
  ctGB->setSon(ctWindow);

  //show root display window but hide current display window for now
  rtGB->show();
  ctGB->hide();

  //initialize vertsplit (the central widget) to contain both the transformation display window and the console
  vertsplit = new DVertSplit(this,0,centralwidget,console);

  //initialize the transformation display window (not really the central widget anymore)
  centralwidget = new MyMultiLineEdit(vertsplit,0,this);
  centralwidget->setText(" ");
  centralwidget->setReadOnly(true);
  centralwidget->setMinimumHeight(centralwidget->minimumSizeHint().height());

  //initialize the console
  console = new DVisConsole(vertsplit);
  console->setText("Welcome to Visual COKO!");
  console->setMinimumHeight(console->minimumSizeHint().height());
  
  setCentralWidget(vertsplit);
  vertsplit->setRelativeSizes(100,56);

  this->db = db;

  //get ready to accept user input into parsedArray
  parsedArray = new char* [maxParameters];
  for (int i = 0; i < maxParameters; i++)
    parsedArray[i] = NULL;

  //set member variables _curr_rbList and _break_rbList to be the passed parameters c and b
  _curr_rbList = c;
  _break_rbList = b;

  //initialize three dialogue boxes that can ask the user questions
  q1 = new DYesNoDialogue("Restart?","The program has finished.\nStart it again from the beginning?","Yes","No");

  q2 = new DYesNoDialogue("Restart?","The program being debugged\nhas been started already.\nStart it from the beginning?","Yes","No");

  q3 = new DYesNoDialogue("Exit","The program is running.\nExit anyway?","Yes","No");

  //intialize the stack window which will display the transformation stack
  _stackWindow = new DStackWindow(view, viewIndicies[2], NULL,0,this);
  _stackWindow->setText(" ");
  _stackWindow->setReadOnly(true);
  _stackWindow->hide();

  currStack = NULL;

  //initialize _transList (the transformation list) to be the passed parameter transList
  _transList = transList;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: close
// Parameters:	(bool alsoDelete)
// Returns: bool
//
// Calls the quit() slot.
//
//////////////////////////////////////////////////////////////////////////////

bool MyMainWindow::close( bool alsoDelete ) {
  //bool temp = QMainWindow::close(alsoDelete);
  quit();
  return false;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: dealWithSelection
// Parameters:	(QString* str)
// Returns: void
//
// This method is called when the user typed in a command. This command is
// passed through the QString* parameter str. This string is then parsed by
// the debugger and control is returned to the Debug Controller.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::dealWithSelection(QString* str)
{
  db->parseInput(str->latin1(), parsedArray, maxParameters);
  db->setReadyToGo();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getDisplayRT
// Parameters:	()
// Returns: bool
//
// This method checks to see if the root KOLA tree display window is
// currently being displayed. If so, then it returns true, otherwise false.
//
//////////////////////////////////////////////////////////////////////////////

int MyMainWindow::getDisplayRT() {
  if (view->isItemChecked(viewIndicies[0])) 
    return 1;
  else
    return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setDisplayRT
// Parameters:	(int x)
// Returns: void
//
// This method takes a variable int (x) and will display the root KOLA tree
// display window to true if x == 0. Otherwise, it will hide this window. It
// updates the view menu accordingly.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::setDisplayRT(int x) {
  //Have to use reverse psychology because showRT will reverse whatever I do
  if (x)
    view->setItemChecked(viewIndicies[0], false);
  else {
    view->setItemChecked(viewIndicies[0], true);
  }
  showRT();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: showRT
// Parameters:	()
// Returns: void
//
// This slot will display the root KOLA tree window if it is currently
// hidden, and will hide it if it is currently being displayed. The view
// menu is updated accordingly.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::showRT() {
  if (view->isItemChecked(viewIndicies[0])) {
    rtGB->hide();
    view->setItemChecked(viewIndicies[0], false);
  }
  else {
    rtGB->show();
    view->setItemChecked(viewIndicies[0], true);
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: showItem
// Parameters:	(int index)
// Returns: void
//
// This slot takes an int (index) which is the index of an item in the view
// menu and will display that item if it is currently hidden, and will hide it
// if it is currently being displayed. The view menu is updated accordingly.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::showItem(int index) {
  DScrollWidget* gb = NULL;
  _svl->initIterator();
  // go through each element in the symbol visual display list
  while (_svl->iteratorValue() ||_svl->iteratorWhichList() ) {
    //if the index of that item is equal to the index parameter
    if (_svl->iteratorViewIndex() == index) {
      //set gb to equal the display window of that item
      gb = _svl->iteratorGB();
      break;
    }
    _svl->moveIterator(); //keep moving through the list
   }

  //if it found the item in the list, hide it if displayed and vice versa
  if (gb) {
    if (view->isItemChecked(index)) {
      gb->hide();
      view->setItemChecked(index, false);
    }
    else {
      gb->show();
      view->setItemChecked(index, true);
    }
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: showTransformation
// Parameters:	(int index)
// Returns: void
//
// This slot takes an int (index) which is the index of an item (in
// particular a transformation) in the view menu and will display that
// transformation if it is currently hidden, and will hide it if it is
// currently being displayed. The view menu is updated accordingly.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::showTransformation(int index) {
  MyMultiLineEdit* tempEdit = NULL;
  _transList->initIterator();
  //go through each element in the transformation list
  while (_transList->iteratorValue()) {
    //if the index of that item is equal to the index parameter
    if (_transList->iteratorViewIndex() == index) {
      //set tempEdit to equal the display window of that item
      tempEdit = _transList->iteratorWindow();
      break;
    }
    _transList->moveIterator();
   }

  //if it found the item in the list, hide it if displayed and vice versa
  if (tempEdit) {
    if (view->isItemChecked(index)) {
      tempEdit->hide();
      view->setItemChecked(index, false);
    }
    else {
      tempEdit->show();
      view->setItemChecked(index, true);
    }
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setDisplayCT
// Parameters:	(int x)
// Returns: void
//
// This method takes a variable int (x) and will display the current KOLA
// tree display window to true if x == 0. Otherwise, it will hide this
// window. It updates the view menu accordingly.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::setDisplayCT(int x) {
  //Have to use reverse psychology because showCT will reverse whatever I do
  if (x)
    view->setItemChecked(viewIndicies[1], false);
  else
    view->setItemChecked(viewIndicies[1], true);
  showCT();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getDisplayCT
// Parameters:	()
// Returns: bool
//
// This method checks to see if the current KOLA tree display window is
// currently being displayed. If so, then it returns true, otherwise false.
//
//////////////////////////////////////////////////////////////////////////////

int MyMainWindow::getDisplayCT() {
  if (view->isItemChecked(viewIndicies[1])) 
    return 1;
  else
    return 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: showCT
// Parameters:	()
// Returns: void
//
// This slot will display the current KOLA tree window if it is currently
// hidden, and will hide it if it is currently being displayed. The view
// menu is updated accordingly.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::showCT() {
  if (view->isItemChecked(viewIndicies[1])) {
    ctGB->hide();
    view->setItemChecked(viewIndicies[1], false);
  }
  else {
    ctGB->show();
    view->setItemChecked(viewIndicies[1], true);
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doClear
// Parameters:	()
// Returns: void
//
// This slot gets called if the clear button was pressed. It figures out
// which key number the user wanted the clear the break upon, and enters
// clear <kn> into the console, updates the parsedArray, and returns control
// to the Debug Controller.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::doClear()
{
  clearParsedArray();
  parsedArray[0] = new char[strlen("clear") + 1];
  sprintf(parsedArray[0], "clear");
  int kn = centralwidget->getCurrKN() + 1;

  //convert int version of key number to character
  char* temp = new char[12];
  for (int i = 0; i < 12; i++)
    temp[i] = '\0';
  sprintf(temp, "%d", kn);
  int count = 0;
  for (int i = 0; i < 12; i++)
    if (temp[i] != '\0')
      count++;
  parsedArray[1] = new char[count+1];
  for (int i = 0; i < count; i++)
    parsedArray[1][i] = temp[i];
  parsedArray[1][count] = '\0';

  console->insertAtEnd("clear ");
  console->insertAtEnd(parsedArray[1]);
  console->insertAtEnd("\n");
  
  db->setReadyToGo(); 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doCondition
// Parameters:	()
// Returns: void
//
// This slot gets called if the condition button was pressed. It figures out
// which key number the user wanted the place the conditional clear break
// upon, and enters condition <kn> into the console, updates the parsedArray,
// and returns control to the Debug Controller.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::doCondition()
{ 
  clearParsedArray();
  parsedArray[0] = new char[strlen("condition") + 1];
  sprintf(parsedArray[0], "condition");
  int kn = centralwidget->getCurrKN() + 1;

  //convert int version of key number to character
  char* temp = new char[12];
  for (int i = 0; i < 12; i++)
    temp[i] = '\0';
  sprintf(temp, "%d", kn);
  int count = 0;
  for (int i = 0; i < 12; i++)
    if (temp[i] != '\0')
      count++;
  parsedArray[1] = new char[count+1];
  for (int i = 0; i < count; i++)
    parsedArray[1][i] = temp[i];
  parsedArray[1][count] = '\0';

  console->insertAtEnd("condition ");
  console->insertAtEnd(parsedArray[1]);
  console->insertAtEnd("\n");
  
  db->setReadyToGo();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doBreak
// Parameters:	()
// Returns: void
//
// This slot gets called if the break button was pressed. It figures out
// which key number the user wanted the place the break upon, and enters
// break <kn> into the console, updates the parsedArray, and returns control
// to the Debug Controller.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::doBreak()
{ 
  clearParsedArray();
  parsedArray[0] = new char[strlen("break") + 1];
  sprintf(parsedArray[0], "break");
  int kn = centralwidget->getCurrKN() + 1;

  //convert int version of key number to character
  char* temp = new char[12];
  for (int i = 0; i < 12; i++)
    temp[i] = '\0';
  sprintf(temp, "%d", kn);
  int count = 0;
  for (int i = 0; i < 12; i++)
    if (temp[i] != '\0')
      count++;
  parsedArray[1] = new char[count+1];
  for (int i = 0; i < count; i++)
    parsedArray[1][i] = temp[i];
  parsedArray[1][count] = '\0';

  console->insertAtEnd("break ");
  console->insertAtEnd(parsedArray[1]);
  console->insertAtEnd("\n");

  db->setReadyToGo();

  //DELETE TEMP??? COME BACK LATER.
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: quit
// Parameters:	()
// Returns: void
//
// This slot gets called if the user tried to exit the program (by pressing
// the exit button, by closing the window, etc). This method places the word
// quit on the console and returns control to the debug controller which
// will make sure that the user really wants to quit.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::quit()
{ 
  clearParsedArray();
  parsedArray[0] = new char[strlen("quit") + 1];
  sprintf(parsedArray[0], "quit");
  console->insertAtEnd("quit\n");
  db->setReadyToGo();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doRun
// Parameters:	()
// Returns: void
//
// This slot gets called if the run button was pressed. It places the word
// "run" on the console and returns control to the debug controller which
// which make sure that the user really wanted to restart execution of the
// root transformation.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::doRun()
{ 
  clearParsedArray();
  parsedArray[0] = new char[strlen("run") + 1];
  sprintf(parsedArray[0], "run");
  console->insertAtEnd("run\n");
  db->setReadyToGo();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: toggleStack
// Parameters:	()
// Returns: void
//
// This slot gets called if the stack option in the view menu is selected.
// If the stack window was currently being displayed, it gets hidden.
// Otherwise it gets displayed.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::toggleStack()
{ 
  if (view->isItemChecked(viewIndicies[2])) {
    _stackWindow->hide();
    view->setItemChecked(viewIndicies[2], false);
  }
  else {
    displayStack();
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doStack
// Parameters:	()
// Returns: void
//
// This slot gets called if the stack window is to be refreshed or displayed.
// It places the word "stack" on the console and returns control to the debug
// controller.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::doStack()
{ 
  clearParsedArray();
  parsedArray[0] = new char[strlen("stack") + 1];
  sprintf(parsedArray[0], "stack");
  console->insertAtEnd("stack\n");
  db->setReadyToGo();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: step
// Parameters:	()
// Returns: void
//
// This slot gets called if the step button is pressed. The word "step" is
// written to the console and control is given back to the debug controller.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::step()
{ 
  clearParsedArray();
  parsedArray[0] = new char[strlen("step") + 1];
  sprintf(parsedArray[0], "step");
  console->insertAtEnd("step\n");
  db->setReadyToGo();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: next
// Parameters:	()
// Returns: void
//
// This slot gets called if the next button is pressed. The word "next" is
// written to the console and control is given back to the debug controller.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::next()
{ 
  clearParsedArray();
  parsedArray[0] = new char[strlen("next") + 1];
  sprintf(parsedArray[0], "next");
  console->insertAtEnd("next\n");
  db->setReadyToGo();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: cont
// Parameters:	()
// Returns: void
//
// This slot gets called if the continue button is pressed. The word
// "continue" is written to the console and control is given back to the
// debug controller.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::cont()
{
  clearParsedArray();
  parsedArray[0] = new char[strlen("continue") + 1];
  sprintf(parsedArray[0], "continue");
  console->insertAtEnd("continue\n");
  db->setReadyToGo();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: updatePaintWindows
// Parameters:	(NonType* rtTree, NonType* ctTree, SymbolList* sym,
//               char* execName)
// Returns: void
//
// This takes a pointer to the root KOLA tree (rtTree) and current KOLA tree
// (ctTree) and sets these to be the trees of the root display and current
// display windows. In addition, it passes the SymbolList (sym) and execName
// string to these windows. Finally it repaints them.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::updatePaintWindows(NonType* rtTree, NonType* ctTree, SymbolList* sym, char* execName) {
  rtWindow->setCurrent(rtTree);
  rtWindow->setImportantValues(sym, execName);
  rtWindow->repaint(true);
  ctWindow->setCurrent(ctTree);
  ctWindow->setImportantValues(sym, execName);
  ctWindow->repaint(true);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: displayRBInfo
// Parameters:	(char* contents, int kn, LCSubstring* currLCA, int numStmts,
//               NonType* rtTree, NonType* ctTree, SymbolList* sym,
//               char* execName, CState* s)
// Returns: void
//
// This is the central method of this class. It is called from the breakLoop
// method of the debug controller after it has figured out the values of all
// of the above parameters. The first 4 parameters are used to display the
// text of the current transformation in main window, the next 4 parameters
// are used to display the root and current KOLA trees in their respective
// windows. The final parameter is the current state of the system and is
// used by the setCurrInformation method to find the values of variables.
// After passing all of this information to their respective windows, these
// windows are refreshed.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::displayRBInfo(char* contents, int kn, LCSubstring* currLCA, int numStmts, NonType* rtTree, NonType* ctTree, SymbolList* sym, char* execName, CState* s)
{ 
  currLCArray=currLCA;
  currKN = kn;
  centralwidget->setCurr(kn,currLCArray, numStmts);
  centralwidget->setText(contents);

  setCurrInformation(sym, execName, rtTree, s);
  rtWindow->setCurrent(rtTree);
  rtWindow->setImportantValues(sym, execName);
  rtWindow->repaint(true);
  ctWindow->setCurrent(ctTree);
  ctWindow->setImportantValues(sym, execName);
  ctWindow->repaint(true);

  //refresh stack window if it is being displayed
  if (view->isItemChecked(viewIndicies[2]))
    displayStack();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getParsedArray
// Parameters:	()
// Returns: char**
//
// This method returns the parsedArray variable. It is called from the debug
// controller when it is ready to analyze the input.
//
//////////////////////////////////////////////////////////////////////////////

char** MyMainWindow::getParsedArray() {
  return parsedArray;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insertConsoleText
// Parameters: (const char* toInsert)
// Returns: void
//
// This method takes some text (toInsert) and inserts it at the end of the
// console window.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::insertConsoleText(const char* toInsert)
{
  console->insertAtEnd(toInsert);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: clearParsedArray
// Parameters: ()
// Returns: void
//
// This method deletes every element in the member variable parsedArray and
// frees its corresponding memory.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::clearParsedArray() 
{
  for (int i = 0; i < maxParameters; i++)
    if (parsedArray[i]) {
      delete[] parsedArray[i];
      parsedArray[i] = NULL;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: enableDisable
// Parameters: (int kn)
// Returns: void
//
// This method makes sure that both the break/condition or clear options
// aren't available at the same time. If the current key number has any kind
// of break on it, then the break and condition buttons and commands (in the
// command menu) are disabled while the clear button and command is enabled.
// If the key number has not break, then the reverse is done.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::enableDisable(int kn) 
{
  if (kn == -1)
    kn = currKN;
  
  if ((currBA[kn]) || (currCA[kn])) {
    b4->setEnabled(false);
    b5->setEnabled(false);
    b6->setEnabled(true);
    commands->setItemEnabled(commandsIndicies[3],false);
    commands->setItemEnabled(commandsIndicies[4],false);
    commands->setItemEnabled(commandsIndicies[5],true);
      }
  else {
    b4->setEnabled(true);
    b5->setEnabled(true);
    b6->setEnabled(false);
    commands->setItemEnabled(commandsIndicies[3],true);
    commands->setItemEnabled(commandsIndicies[4],true);
    commands->setItemEnabled(commandsIndicies[5],false);
  }
    
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setCurrInformation
// Parameters:	(SymbolList* sym, char* execName, NonType* rootTree, CState* s)
// Returns: void
//
// This method updates the important variables in every paint window that
// displays the KOLA trees and the central widget (textual transformation
// display) window. It deletes windows that are no longer useful and
// refreshes the ones that are.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::setCurrInformation(SymbolList* sym, char* execName, NonType* rootTree, CState* s)
{
  int temp = db->findCurrent(NULL);
  if (temp == 0) {
    cout << "Error: current does not exist." << endl;
    exit(1);
  }
  else {
    
    //update the break array, the condition array, and the array sizes
    //I can use iteratorBA and iteratorCA directly because I know they have
    //been set in the debug controller and _break_rbList points to the same
    //thing as _break_rbList in the debug controller
    currBA = _break_rbList->iteratorBA();
    currCA = _break_rbList->iteratorCA();
    currArraySize = _break_rbList->iteratorArraySize();
    centralwidget->updateBCA(currBA,currCA,currArraySize);
  }

  //enable/disable menu/button options depending on the values of currBA and currCA 
  enableDisable(centralwidget->getCurrKN());

  DPaintWindow* tempPW;
  DWhichList* tempWL;
  _svl->initIterator();

  //go through the visual symbol list and update the important values in each
  //of the corresponding paint windows.
  while (_svl->iteratorValue() || _svl->iteratorWhichList()) {

    //delete nodes/group boxes that are marked deleted
    if ((_svl->iteratorGB()) && (_svl->iteratorGB()->getMarkedDeleted())) {
      _svl->setIteratorNum(-1);
      _svl->delete_marked_nodes();
      _svl->initIterator();
    }
    
    //otherwise get the appropriate paint window and update the important values
    else {
      tempPW = _svl->iteratorPW();
      tempWL = _svl->iteratorWhichList();
      tempPW->setImportantValues(sym, execName);

      //if a which list exists then we must be looking at an isolation - so
      //recalculate the isolation from its root tree
      if (tempWL)
	tempPW->setCurrent(doIsolate(rootTree,tempWL)); 
      else { //looking at a variable display window
	CIdent* myNode;
	if (myNode = s->IFindDeepDB(_svl->iteratorValue()))
	  tempPW->setCurrent(myNode->Child_Look_Up()); //find the variable val
	else
	  _svl->setIteratorNum(-1); // if not found then delete node
      }  
      tempPW->repaint(true); //repaint that window
      _svl->iteratorGB()->show();
      _svl->moveIterator();
    }
  }
  //repaint each open window in case a break was added/removed
  centralwidget->repaint();
  _transList->initIterator();
  MyMultiLineEdit* tempEdit;
  if (tempEdit = _transList->iteratorWindow()) //add check view
    tempEdit->repaint();
  if (tempEdit = _transList->iteratorNextWindow()) //add check view
    tempEdit->repaint();
  while (_transList->getNextValue())
    if (tempEdit = _transList->iteratorNextWindow()) //add check view
      tempEdit->repaint();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: checkDoAgain
// Parameters: (int which)
// Returns: int
//
// This method takes an integer parameter (which) that specifies which of the
// three dialog boxes to display. The corrosponding dialog box is then
// the displyed, and the user response is returned.
//
//////////////////////////////////////////////////////////////////////////////

int MyMainWindow::checkDoAgain(int which)
{
  DYesNoDialogue* q;
  if (which == 1)
    q = q1;
  else if (which == 2)
    q = q2;
  else if (which == 3)
    q = q3;
  //Because DYesNoDialogue is a modal dialog,
  //we call exec() instead of show(): 
  while (true) {
    int ans = q->exec();
    
    if( ans == QDialog::Accepted )
      {
	return 1;
      }
    else if( ans == QDialog::Rejected )	
      {
	return 0;
      }
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: setSVL
// Parameters: (DSymbolVisList* svl)
// Returns: void
//
// This method sets the member variable _svl to be equal to the symbol visual
// display list parameter (svl)
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::setSVL(DSymbolVisList* svl) {
  _svl = svl;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insertIntoView
// Parameters: (char* toInsert)
// Returns: int
//
// This method inserts into the view menu an new item labeled by the
// parameter: toInsert. It then connects this item SIGNAL to the SLOT:
// showItem. Finally, it returns the index of this new inserted item in the
// view menu.
//
//////////////////////////////////////////////////////////////////////////////

int MyMainWindow::insertIntoView(char* toInsert) {
  int temp = view->insertItem(toInsert);
  view->connectItem(temp, this, SLOT(showItem(int)));
  view->setItemChecked(temp, true);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: removeFromView
// Parameters: (int index)
// Returns: void
//
// This method removes from the view menu the item with the index
// corresponding to the int parameter: index.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::removeFromView(int index) {
  view->removeItem(index);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getView
// Parameters: ()
// Returns: QPopupMenu*
//
// This method returns the view popupmenu stored in member variable: view
//
//////////////////////////////////////////////////////////////////////////////

QPopupMenu* MyMainWindow::getView() {
  return view;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doIsolate
// Parameters: (NonType* rootTree, DWhichList* wl)
// Returns: NonType*
//
// This method refreshes the KOLA tree isolation starting from a particular
// root tree (rootTree) using a particular Which List (wl). Each element
// in wl is an integer indicating how to travese the root tree from the
// given point by telling it which child to follow. For instance, if the
// which list contains: 2 1 1, then this means that the isolation is the
// second child of the first child of the first child of the root tree. This
// method returns the tree derived from this isolation algorithm.
//
//////////////////////////////////////////////////////////////////////////////

NonType* MyMainWindow::doIsolate(NonType* rootTree, DWhichList* wl)
{
  wl->initIterator();
  int which = 0;
  NonType* child;
  //iterate through the Which List
  while (which = wl->iteratorValue()) {
    rootTree->initCurrChild();
    //iterate through the children of this node "which" times
    while ((!((child=rootTree->nextChild()) == NULL)) && (which > 1)) {
      which--;
    }
    //when we stop the while loop, child is the correct ancestor of the
    //branch that has been isolated. So do the make child the parent and
    //so the same thing with the next which number from the which list
    rootTree = child;
    wl->moveIterator();
  }
  if (rootTree)
    return rootTree;
  return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insertIntoSVL
// Parameters: (char* toInsert, DWhichList* wl, int displayCounter,
//              DScrollWidget* gb, DPaintWindow* pw, int index)
// Returns: void
//
// This method simply passes along these parameters to the insert method of
// the symbol visual display list. This list contains the windows of all of
// the additional windows besides the root KOLA tree and current KOLA tree
// display windows (hence isolations and variable display windows).
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::insertIntoSVL(char* toInsert, DWhichList* wl, int displayCounter, DScrollWidget* gb, DPaintWindow* pw, int index) {
  _svl->insert(toInsert, wl, displayCounter, gb, pw, index);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: displayStack
// Parameters: ()
// Returns: void
//
// This method recalculates what should be in the stack window by calling the
// displayStack method on the _curr_rbList and then sets the text of the
// stack display window to be this returned text. The stack window is then
// display and the view menu is updated accordingly.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::displayStack()
{
  if (currStack)
    delete[] currStack;
  currStack = _curr_rbList->displayStack();
  _stackWindow->setText(currStack);
  _stackWindow->show();
  view->setItemChecked(viewIndicies[2], true);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: doExpandTrans
// Parameters:	()
//
// This slot is called from the popup menu derived from right-clicking on
// the stack window. It will put the words "track <name-of-trans> to the
// console and allow the debugger to execute the command.
//         								    
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::doExpandTrans()
{
  int lineNum = _stackWindow->getCurrLine(); //figure out which line the user right clicked on (this is the transformation that the user wants to expand)
  clearParsedArray();
  parsedArray[0] = new char[strlen("track") + 1];
  sprintf(parsedArray[0], "track");
  parsedArray[1] = new char[strlen(_stackWindow->textLine(lineNum)) + 1];
  sprintf(parsedArray[1], _stackWindow->textLine(lineNum));
  //parsedArray should now contain "track <name-of-trans>"

  console->insertAtEnd("track ");
  console->insertAtEnd(parsedArray[1]);
  console->insertAtEnd("\n");
  
  db->setReadyToGo(); 
  
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: expandTrans
// Parameters:	(char* (optional))
//
// This method will expand a transformation (meaning it will open a new
// window with the transformation code). Which transformation is expanded can
// be indicated in two ways. The first way is by simply passing the name of
// the transformation to the method (parameter transName). The other way is
// by double clicking the appropriate line in the stack window which this
// method can figure out by seeing which line is active in the stack display
// window.
//         								    
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::expandTrans(char* transName = NULL)
{
  int inList, lineNum;
  _curr_rbList->initIterator();
  if (!transName) { //transformation indicated in stack window
    lineNum = _stackWindow->getCurrLine(); //figure out which line was clicked
    if (lineNum) 
      for (int i = 0; i < lineNum; i++) //go to the correct node in _curr_rbList
	_curr_rbList->iteratorNext();
    inList = _transList->find(_stackWindow->textLine(lineNum)); //check to see if the indicated transfomration is already being displayed.
  }
  else { //transName passed to method (user likely typed it in)
    while ((_curr_rbList->iteratorName()) && (strcmp(_curr_rbList->iteratorName(), transName) != 0))
      _curr_rbList->iteratorNext();
    if (_curr_rbList->iteratorName()) //if transName is in _curr_rbList
      inList = _transList->find(transName); //check to see if the indicated transfomration is already being displayed.
    else
      inList = 0;
  }
   
  if (!inList) {
    int temp = db->findCurrent(_curr_rbList->iteratorValue(), NULL); //get _break_rbList pointing to correct transformatoion
    if ((temp == 0) || (!(_curr_rbList->iteratorValue())))
      console->insertAtEnd("ERROR: Stack element does not exist.\n");
    else { //create new window for transformation
      MyMultiLineEdit* tempEdit = new MyMultiLineEdit(NULL,0,this,1);
      tempEdit->setReadOnly(true);
      char* rbContents = _curr_rbList->iteratorValue()->representation_with_debugging_info(db); //transfomration contents are easily gotten since _curr_rbList is already pointing to the correct node
      tempEdit->setText(rbContents);
      tempEdit->setCurr(1,db->getCurrLCArray(), db->getNumStmts());
      tempEdit->updateBCA(_break_rbList->iteratorBA(),_break_rbList->iteratorCA(),_break_rbList->iteratorArraySize()); //associate the correct breakArray and conditionArray with the new window
      int index;
      //insert transfomration window into view menu
      if (transName)
	index = view->insertItem(transName);
      else
	index = view->insertItem(_stackWindow->textLine(lineNum));
      view->connectItem(index, this, SLOT(showTransformation(int)));
      view->setItemChecked(index, true);
      
      //I realize now that I probably don't need the third parameter (the char* transfomration code contents) but I think I will leave it in because it might be useful to have in the future.
      _transList->insert(_break_rbList->getIterator(), tempEdit, rbContents, index); //add transfomration to list of viewable transformations
      console->insertAtEnd("Transfomation ");
      console->insertAtEnd(transName);
      console->insertAtEnd(" placed in tracking list.\n");
      tempEdit->show();
      tempEdit = NULL;
    }
  }
  else if (inList == 1){
    showTransformation(_transList->iteratorNextViewIndex());
    console->insertAtEnd("Transfomation ");
    console->insertAtEnd(transName);
    console->insertAtEnd(" is already being displayed.\n");
  }
  else if (inList == 2) {
    showTransformation(_transList->iteratorViewIndex());
    console->insertAtEnd("Transfomation ");
    console->insertAtEnd(transName);
    console->insertAtEnd(" is already being displayed.\n");
  }
    
  int temp = db->findCurrent(NULL);
  if (temp == 0) {
    cout << "Error: current does not exist." << endl;
    exit(1);
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: dbGo
// Parameters: ()
// Returns: void
//
// This method simply tells the debug controller that it is ok to regain
// control by calling its setReadyToGo method.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::dbGo()
{
  db->setReadyToGo();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: removeFromTransList
// Parameters:	(char*)
// Returns: void
//
// If the user closed a transformation window, we assume that the user no
// longer is interested in it, so it gets removed from the list of viewable
// transformations.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::removeFromTransList(char* transName)
{
  int inList = _transList->find(transName);
  int index;
  if (inList == 1) {
    index = _transList->iteratorNextViewIndex();
    _transList->deleteNext();
  }
  else if (inList == 2) {
    index = _transList->iteratorViewIndex();
    _transList->pop();
  }
  if (inList) //remove from view menu as well
    removeFromView(index);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: closeAllSVLWindows
// Parameters:	()
// Returns: void
//
// This method goes though each element in the symbol visual display list
// (_svl) and closes and deletes each window.
//
//////////////////////////////////////////////////////////////////////////////

void MyMainWindow::closeAllSVLWindows()
{
  DPaintWindow* tempPW;
  DWhichList* tempWL;
  _svl->initIterator();

  //go through the visual symbol list and close and delete each window
  while (_svl->iteratorValue() || _svl->iteratorWhichList()) {

    //delete all nodes
    if (_svl->iteratorGB()) {
      if (!(_svl->iteratorWhichList())) {
	int index = _svl->iteratorViewIndex();
	removeFromView(index);
    }
      _svl->iteratorGB()->close(false);
      _svl->setIteratorNum(-1);
    } 
    _svl->moveIterator();
  }
  _svl->delete_marked_nodes();
}
