//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon Suk Lee (jl) 
//  FILE: NonTypeList.C
//  PROJ: COKO
//  DATE: 10/97                                                              
//
// SYNOPSIS
//  Classes for NonType List.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef _NONTYPELIST_C_
#define _NONTYPELIST_C_

#include "NonTypeList.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: NonTypeNode
// Parameters:	(char*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

NonTypeNode::NonTypeNode(NonType * a):	
  _next(NULL),
  _value(NULL)
{
  _value = a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~NonTypeNode
// Parameters:	(void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////
  
NonTypeNode::~NonTypeNode(void) 
{
  if (_value) { delete _value; _value = NULL; }
  _next = NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: insert
// Parameters: (char*)	
//
// Insert a new node with _value = a to the list.
//         								    
//////////////////////////////////////////////////////////////////////////////

void NonTypeList::insert(NonType* a) 
{
  if (!_head)
	_head = _tail = new NonTypeNode(a); 
  else 
	{
	  NonTypeNode * tmp = new NonTypeNode(a);
	  _tail->_next = tmp;
	  _tail = tmp;
	  tmp = NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: pop
// Parameters: (void)
//
// Pop off a front node from the list.
// Return success value of the pop.
//         								    
//////////////////////////////////////////////////////////////////////////////

int NonTypeList::pop(void)
{
  if (!_head)
	return NULL;

  NonTypeNode * tmpnode = _head;

  if (_head == _tail)
	_head = _tail = NULL;
  else
	_head = _head->_next;

  if (tmpnode->_value) 
	{ delete tmpnode; tmpnode = NULL; }
  else
	{
	  cout << "Error @ NonTypeList : pop " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return NULL;
	}

  return 1;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: top
// Parameters: (NonType*)
//
// Return _value of the top node.
//         								    
//////////////////////////////////////////////////////////////////////////////

NonType* NonTypeList::top(void)
{
  if (!_head)
	return NULL;

  if (_head->_value) 
	return _head->_value;
  else
	{
	  cout << "Error @ NonTypeList : top " << endl;
	  cout << "node has no content error " << endl;
	  exit(1);
	  return NULL;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: empty
// Parameters: (void)
//
// Is the list empty?
//         								    
//////////////////////////////////////////////////////////////////////////////

int NonTypeList::empty(void) 
{ 
  return (!_head); 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: initIterator 
// Parameters: (void)
//
// Initialize _iterator.
// This method is used with getValue method.
//         								    
//////////////////////////////////////////////////////////////////////////////

void NonTypeList::initIterator(void)
{
  _iterator = _head;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: iteratorValue
// Parameters:	(void)
//
// Returns _value of the node that _iterator is pointing to.
//
//////////////////////////////////////////////////////////////////////////////

NonType* NonTypeList::iteratorValue(void)
{
  if (_iterator)
	return _iterator->_value;
  else
	return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: getValue
// Parameters:	(void)
//
// Returns _value of the node that _iterator is pointing to.
// After a call to this method, _iterator will point to the next node.
// One can call this methods repeatly to get the _value of all the nodes.
// However, initIterator has to be called before the initial call to this 
// method.  
//         								    
//////////////////////////////////////////////////////////////////////////////

NonType* NonTypeList::getValue()
{
  NonType* return_val;
  if (_iterator)
	{
	  return_val = _iterator->_value;
	  _iterator = _iterator->_next;
	}
  else
	return_val = NULL;
  return return_val;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: NonTypeList
// Parameters: (void)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

NonTypeList::NonTypeList():
  _head(NULL),
  _tail(NULL),
  _iterator(NULL)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:~NonTypeList
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

NonTypeList::~NonTypeList()
{
  NonTypeNode * tmp = _head;
  _tail = NULL;
  while (_head)
   	{
 	  _head = _head->_next;
 	  delete tmp;
 	  tmp = _head;
 	}
}

#endif

