//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon S. Lee (jl@cs.brown.edu)
//  FILE: PCFalseStmt.C
//  PROJ: COKO
//  DATE: 6/10/97                                                              
//
// SYNOPSIS
//  Class definition for CFalseStmt class.
//  
//////////////////////////////////////////////////////////////////////////////

#include <assert.h>
#include "PCState.H"
#include "PCStmt.H"
#include "PCFalseStmt.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CFalseStmt
// Parameters: (CStmt*)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CFalseStmt::CFalseStmt(CStmt* stmt)
{
  _stmt = stmt;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CFalseStmt
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CFalseStmt::~CFalseStmt(void)
{
  if(_stmt) { delete _stmt; _stmt = NULL;}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters: (CState*)
//
// What to do when executed
// Execute _stmt then set _Success to false.
//
//////////////////////////////////////////////////////////////////////////////

CState* CFalseStmt::Exec(CState *s)
{
  s = _stmt->Exec(s);

  // set _Success to False 
  s->SetSuccess(F);

  return s;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ExecDebug
// Parameters: (CState*, DDebugController*)
//
// What to do when executed
// Execute _stmt then set _Success to false.
//
//////////////////////////////////////////////////////////////////////////////

CState* CFalseStmt::ExecDebug(CState *s, DDebugController *db)
{
  if (db->really_execute()) {
    s = _stmt->ExecDebug(s, db);
    
    // set _Success to False 
    s->SetSuccess(F);
  }
    return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CFalseStmt::rep_string ()
{
  char *res = new char [strlen ("FALSEv") + 1];
  sprintf (res, "FALSEv");
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CFalseStmt::representation (int indent)
{
  char *stmt;
  if (_stmt -> is_leaf())
    stmt = _stmt -> representation (indent);
  else {
    char *stmt_orig = _stmt -> representation (indent);
    stmt = new char[strlen(stmt_orig) + strlen("{}") + 1];
    sprintf (stmt, "{%s}", stmt_orig);
    delete[] stmt_orig;
  }
  char *rep = rep_string();
  char *res = new char [strlen (stmt) + strlen (rep) + strlen (" ") + 1];
  sprintf (res, "%s %s", rep, stmt);
  delete [] stmt;
  delete [] rep;
  return res; 
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController *)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CFalseStmt::representationWDI (int indent, DDebugController *db)
{
  char *rep = rep_string();
  int currCol = db->getCol();
  db->setCol(currCol + strlen(rep) + 1);
  char *stmt;
  if (_stmt -> is_leaf())
    stmt = _stmt -> representation (indent);
  else {
    db->setCol(currCol+strlen(rep)+2);
    char *stmt_orig = _stmt -> representation (indent);
    stmt = new char[strlen(stmt_orig) + strlen("{}") + 1];
    sprintf (stmt, "{%s}", stmt_orig);
    delete[] stmt_orig;
    currCol = db->getCol();
    db->setCol(currCol+1);
  }
  char *res = new char [strlen (stmt) + strlen (rep) + strlen (" ") + 1];
  sprintf (res, "%s %s", rep, stmt);
  delete [] stmt;
  delete [] rep;
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: number_self
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Uses start parameter to assign key_number data_member, calls number_self 
// on any sub_statements and returns current key-number
//         								    
//////////////////////////////////////////////////////////////////////////////

int CFalseStmt::number_self (int start)
{
  key_number=0; //FalseV not a breakable statement
  return (_stmt -> number_self(start));
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: output_self
// Parameters:	(DDebugController *)
//
// ADDED DJA SUMMER 2001
// Outputs current statement in list format (for the debugger) if it is a 
// breakable statement; otherwise just calls output_self on any sub-statements
//         								    
//////////////////////////////////////////////////////////////////////////////

void CFalseStmt::output_self (DDebugController *db)
{
  //FALSEV is not a breakable statment
  _stmt->output_self(db);
}

int CFalseStmt::is_leaf()
//returns whether current statement is a leaf in the COKO tree
{
  //I know that FALSEV isn't really a leaf, but for the purposes of the
  //representation method which uses is_leaf, FALSEV is just one statement
  //like a leaf
  return 1;
}
