//////////////////////////////////////////////////////////////////
//
//  NAME: Joon S. Lee (jl)
//  
//  FILE: PCRule.C
//  PROJ: COKO
//  DATE: 6/96
// 
//  SYNOPSIS
//  This class is for Rule Declaration/Invoke statements.
//
/////////////////////////////////////////////////////////////////
#include "PCStmt.H"
#include "PCRule.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CRule (CPattern* cp1, CPattern* cp2)
// Parameters: (CPattern*, CPattern*)
//
//  Constructor for rule representation. 
//  Takes two arguments that are patterns of
//  lhs and rhs, respectively. 
//         								    
//////////////////////////////////////////////////////////////////////////////

CRule::CRule (CPattern* cp1, CPattern* cp2)
{
  _name = NULL;
  _lhs = cp1;
  _rhs = cp2;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CRule (char* name, CPattern* cp1, CPattern* cp2)
// Parameters: (char*, CPattern*, CPattern*)
//
//  Constructor for rule representation. 
//  Takes three arguments that are rule name, and
//  patterns of lhs and rhs, respectively. 
//         								    
//////////////////////////////////////////////////////////////////////////////

CRule::CRule (char* name, CPattern* cp1, CPattern* cp2)
{
  if (name) {
    _name = new char[strlen(name) + 1];
    strcpy(_name, name);
  }
  else
    _name = NULL;
  _lhs = cp1;
  _rhs = cp2;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CRule 
// Parameters: (void)
//
// Destructor
//         								    
//////////////////////////////////////////////////////////////////////////////

CRule::~CRule(void)
{
  if(_lhs) { delete _lhs; _lhs = NULL; }
  if(_rhs) { delete _rhs; _rhs = NULL; }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ExecDebug
// Parameters: (CState*, DDebugController *db)
//
//  Executes the given statement in debugger mode.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState* CRule::ExecDebug(CState* s, DDebugController *db, CStmt* ris)
{
  CPattern *old_pattern = s->CurrentPattern();
  if (db->really_execute()) {
    CState* resultState;
    CState* olds;
    if (db->ruleOn()) {
      olds = s;
      db->printInRightPlace("------------------------------------------------------");
      db->printInRightPlace("\n");
      db->printInRightPlace("ATTEMPT TO EXECUTE RULE:");
      db->printInRightPlace("\n");
      db->printInRightPlace(_lhs->representation());
      db->printInRightPlace("-->");
      db->printInRightPlace(_rhs ->representation());
      db->printInRightPlace("\n");
      db->printInRightPlace("on tree: ");
      db->printInRightPlace(olds->Store()->representation());
      db->printInRightPlace("\n");
    }
    
    resultState = _lhs->match(s);
    if(s->ISuccess()) {
      if (db->check_for_break(ris->get_key_number())) {
    	resultState->SetCurrentPattern(_lhs);
    	db->doBreak(ris, resultState);
    	resultState->SetCurrentPattern(old_pattern);
      }
    }
    
    if(s->ISuccess()) {
      resultState = _rhs->build(resultState);
      if (db->ruleOn()) {
	db->printInRightPlace("ATTEMPT SUCCEEDED:");
	db->printInRightPlace("\n");
	db->printInRightPlace("Result State is: ");
	db->printInRightPlace(resultState->Store()->representation());
	db->printInRightPlace("\n");
	db->printInRightPlace("------------------------------------------------------");
	db->printInRightPlace("\n");
      }
    }
    else {
      if (db->ruleOn()) {
	db->printInRightPlace("Attempt failed.");
	db->printInRightPlace("\n");
	db->printInRightPlace("------------------------------------------------------");
	db->printInRightPlace("\n");
      }
    }
    
    return resultState;
  }
  else
    return s;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters: (CState*)
//
//  Executes the given statement.
//  Takes the state of the program before the statement
//  is executed, and then returns the state of the 
//  program after the statement is executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState* CRule::Exec(CState* s)
{

  if(_debug_)
	cout << "-- CRule Begins" << endl;

  CState* resultState;
  CState* olds;

  if(_cldbg_) {
    olds = s;
    cout << "------------------------------------------------------" << endl;
    cout << "ATTEMPT TO EXECUTE RULE:" << endl;
    cout << _lhs->representation() << "-->" << _rhs ->representation() << endl;
    cout << "on tree: " << olds->Store()->representation() << endl;
  }

  resultState = _lhs->match(s);

  if(s->ISuccess()) {
    resultState = _rhs->build(resultState);
    if(_cldbg_) {
      cout << "ATTEMPT SUCCEEDED:" << endl;
      cout << "Result State is: " << resultState->Store()->representation() << endl;
      cout << "------------------------------------------------------" << endl;
    }
  }
  else {
    if (_cldbg_) {
      cout << "Attempt failed." << endl;
      cout << "------------------------------------------------------" << endl;
    }
  }


  if(_debug_)
    cout << "-- CRule Ends" << endl;
  
  return resultState;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: InvExecDebug
// Parameters: (CState*, DDebugController *db)
//
//  Executes the given statement "in reverse". Same as InvExec except for 
//  debugger
//         								    
//////////////////////////////////////////////////////////////////////////////

CState* CRule::InvExecDebug(CState* s, DDebugController *db, CStmt* ris)
{

  if (db->really_execute()) {
    CState* resultState;
    CState* olds;
    
    if (db->ruleOn()) {
      olds = s;
      db->printInRightPlace("------------------------------------------------------");
      db->printInRightPlace("\n");
      db->printInRightPlace("ATTEMPT TO EXECUTE RULE:");
      db->printInRightPlace("\n");
      db->printInRightPlace(_rhs->representation());
      db->printInRightPlace("-->");
      db->printInRightPlace(_lhs ->representation());
      db->printInRightPlace("\n");
      db->printInRightPlace("on tree: ");
      db->printInRightPlace(olds->Store()->representation());
      db->printInRightPlace("\n");
    }
    resultState = _rhs->match(s);
    if(s->ISuccess()) {
      if (db->check_for_break(ris->get_key_number())) {
	resultState->SetCurrentPattern(_rhs);
	db->doBreak(ris, resultState);
      }
    }
    
    if(s->ISuccess()) {
      resultState = _lhs->build(resultState);
      if (db->ruleOn()) {
	db->printInRightPlace("ATTEMPT SUCCEEDED:");
	db->printInRightPlace("\n");
	db->printInRightPlace("Result State is: ");
	db->printInRightPlace(resultState->Store()->representation());
	db->printInRightPlace("\n");
	db->printInRightPlace("------------------------------------------------------");
	db->printInRightPlace("\n");
      }
    } 
    return resultState;
  }
  else {
    if (db->ruleOn()) {
      db->printInRightPlace("Attempt failed.");
      db->printInRightPlace("\n");
      db->printInRightPlace("------------------------------------------------------");
      db->printInRightPlace("\n");
    }
    return s;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: InvExec
// Parameters: (CState*)
//
//  Executes the given statement "in reverse".
//  Takes the state of the program before the statement
//  is executed, and then returns the state of the 
//  program after the statement is executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState* CRule::InvExec(CState* s)
{
  if(_debug_)
	cout << "-- CRule (Inv) Begins" << endl;

  CState* resultState;
  resultState = _rhs->match(s);
  
  if(s->ISuccess()) resultState = _lhs->build(resultState);
  
  if(_debug_)
	cout << "-- CRule (Inv) Ends" << endl;

  return resultState;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: get_name
// Parameters:	()
//
// ADDED DJA SUMMER 2001
// Returns a copy of the value stored in the _name data member
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRule::get_name()
{
  if (_name)
    {
      char* name = new char[strlen(_name)+1];
      strcpy(name, _name);
      return name;
    }
  else
    return NULL;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: get_rep
// Parameters:	()
//
// ADDED DJA SUMMER 2001
// This method will cause the rule to ALWAYS return the rule representation
// even if the name of the rule is stored.
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRule::get_rep ()
{ 
  char *left = _lhs->representation();
  char *right = _rhs->representation();
  char *res = new char [strlen (left) + strlen (right) + strlen (" --> ") + 1];
  sprintf (res, "%s --> %s", left, right);
  delete [] left;
  delete [] right;
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRule::rep_string () 
{
  if (_name)
    return get_name();
  else {
    char *left = _lhs->representation();
    char *right = _rhs->representation();
    char *res = new char [strlen (left) + strlen (right) + strlen ("[ --> ]") + 1];
    sprintf (res, "[%s --> %s]", left, right);
    delete [] left;
    delete [] right;
    return res; 
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	()
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRule::representation ()
{
  return rep_string ();
}
