//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Daniel Abadi                                            
//  FILE: PCRuleBlock.C
//  PROJ: COKO
//  DATE: 6/26/01                                                              
//
// SYNOPSIS
//  Class definition for CRuleBlock class.
//  
//////////////////////////////////////////////////////////////////////////////

#include "PCStmt.H"
#include "PCRuleBlock.H"

//////////////////////////////////////////////////////////////////////////////
//
//  CLASS: CRuleBlock
//
// SYNOPSIS
//  This class is an abstract base class for rule blocks. When the COKO parser
//  generates C++ code for a rule block, it will make the rule block a
//  derived class of CRuleBlock.
//
//////////////////////////////////////////////////////////////////////////////

CState* CRuleBlock::Exec(CState *s) { return s; }  // It does nothing by default;

CState* CRuleBlock::ExecDebug(CState *s, DDebugController *db) { return s; } 
// It does nothing by default;


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: get_name
// Parameters:	(void)
//
// ADDED DJA SUMMER 2001
// Default method for getting the name of the current RuleBlock. This should
// ALWAYS be overridden.
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRuleBlock::get_name() 
{ 
  char *rb_name = new char [strlen ("RULEBLOCK") + 1];
  sprintf (rb_name, "RULEBLOCK");
  return rb_name;  
} //name is RULEBLOCK by default


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: construct_stmt
// Parameters:	(void)
//
// ADDED DJA SUMMER 2001
// Default method for constructing the main statement for the current
// RuleBlock. This should ALWAYS be overridden.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CRuleBlock::construct_stmt() { _rStmt_ = NULL; }


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: get_stmt
// Parameters:	(void)
//
// ADDED DJA SUMMER 2001
// Returns the value stored in the _rStmt_ data member. This is the statment
// The the Exec or ExecDebug method will execute.
//         								    
//////////////////////////////////////////////////////////////////////////////

CStmt* CRuleBlock::get_stmt() {return _rStmt_;}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: number_self
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Assign an integer to each statement in the current rule-block for command
// line debugger reference especially in the list command.
//         								    
//////////////////////////////////////////////////////////////////////////////

int CRuleBlock::number_self(int start)
{
  numbered_self = 1;
  if (constructed_stmt)
    return (_rStmt_ -> number_self(start));
  else
    return start;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: output_self
// Parameters:	(DDebugController *)
//
// ADDED DJA SUMMER 2001
// Output the header Key Number | Statement and then underneath output each
// breakable statement in this rule block along with its corresponding
// rep_string.
//         								    
//////////////////////////////////////////////////////////////////////////////

void CRuleBlock::output_self(DDebugController *db)
{
  db -> format_output("Key Number", "Statement");
  if ((numbered_self) || (constructed_stmt))
    _rStmt_ -> output_self(db);
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	()
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree stored in this RuleBlock
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRuleBlock::representation ()
{ 
  char *rep = _rStmt_ -> representation (4);
  return rep_with_wrapper(rep);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_with_wrapper
// Parameters:	(char*)
//
// ADDED DJA SUMMER 2001
// Takes the maincontent of the ruleblcok and adds USES section, BEGIN
// END, etc. and returns string representation of COKO tree stored in this 
// RuleBlock
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRuleBlock::rep_with_wrapper (char* rep)
{ 
  char *indent = _rStmt_ -> get_indent_string(4);
  char *name = get_name();
  char *rule_name;
  char *rule_content;
  char *uses;
  char *uses_copy = NULL;
  CRule *rule;
  CRuleBlock *rb;
  _useList->initIterator();
  while ((rule = _useList->iteratorRule()) || (rb = _useList->iteratorRuleBlock())) {
    if (rule) {
      rule_name = rule->get_name();
      rule_content = rule->get_rep();
      if (uses_copy) {
	uses = new char[strlen(uses_copy) + strlen(rule_name) + strlen(rule_content) + strlen(",\n    : ") + 1];
	sprintf(uses, "%s,\n    %s: %s", uses_copy, rule_name, rule_content);
	delete[] uses_copy;
	uses_copy = new char[strlen(uses) + 1];
	strcpy(uses_copy, uses);
	delete[] uses;
	delete[] rule_name;
	delete[] rule_content;
      }
      else {
	uses = new char[strlen(rule_name) + strlen(rule_content) + strlen("    : ") + 1];
	sprintf(uses, "    %s: %s", rule_name, rule_content);
	uses_copy = new char[strlen(uses) + 1];
	strcpy(uses_copy, uses);
	delete[] uses;
	delete[] rule_name;
	delete[] rule_content;
      }
    }
    else {
      if (uses_copy) {
	rule_name = rb->get_name();
	uses = new char[strlen(uses_copy) + strlen(rule_name) + strlen(",\n    ") + 1];
	sprintf(uses, "%s,\n    %s", uses_copy, rule_name);
	delete[] uses_copy;
	uses_copy = new char[strlen(uses) + 1];
	strcpy(uses_copy, uses);
	delete[] uses;
	delete[] rule_name;
      }
      else {
	rule_name = rb->get_name();
	uses_copy = new char[strlen(rule_name) + strlen("    ") + 1];
	sprintf(uses_copy, "    %s", rule_name);
	delete[] rule_name;
      }
    }
    
    _useList->getValue();
  }
  if (uses_copy) {
    char *res = new char [strlen (rep) + strlen(name) + strlen(uses_copy) + strlen(indent) + strlen ("TRANSFORMATION \n  USES\n\n  BEGIN\n\n  END") + 1];
    sprintf (res, "TRANSFORMATION %s\n  USES\n%s\n  BEGIN\n%s%s\n  END", name, uses_copy, indent, rep);
    delete [] rep;
    delete [] name;
    delete [] uses_copy;
    return res;
  }
  else {
    char *res = new char [strlen (rep) + strlen(name) + strlen(indent) + strlen ("TRANSFORMATION \n  BEGIN\n\n  END") + 1];
    sprintf (res, "TRANSFORMATION %s\n  BEGIN\n%s%s\n  END", name, indent, rep);
    delete [] rep;
    delete [] name;
    return res;
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation_with_debugging_info
// Parameters:	(DDebugController *db)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
// 
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRuleBlock::representation_with_debugging_info (DDebugController *db)
{ 
  CRule *rule;
  CRuleBlock *rb;
  int count = 0;
  _useList->initIterator();
  while ((rule = _useList->iteratorRule()) || (rb = _useList->iteratorRuleBlock())) {
    _useList->getValue();
    count++;
  }
  if (count == 0)
    count--;
  db -> repInitialize(3+count, 4, get_num_stmts());
  char *rep = _rStmt_->representationWDI (4,db);
  return rep_with_wrapper(rep);
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: get_num_stmts
// Parameters:	()
//
// ADDED DJA SUMMER 2001
// Return the value stored in the num_stmts data member
//         								    
//////////////////////////////////////////////////////////////////////////////

int CRuleBlock::get_num_stmts ()
{
  if (!numbered_self)
    num_stmts = number_self(1) - 1;
  return num_stmts;
}
