//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon S. Lee (jl), Kee-Eung Kim (kek)  
//  FILE: PCRuleInvokeStmt.C
//  PROJ: COKO
//  DATE: 5/5/97
//
// SYNOPSIS
//  Method definitions for CRuleInvokeStmt class.
//  
//////////////////////////////////////////////////////////////////////////////

#include <assert.h>
#include "PCStmt.H"
#include "PCState.H"
#include "PCRuleInvokeStmt.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CRuleInvokeStmt
// Parameters: (CRule*, char*)
//
// Constructor for rule invoke.
//         								    
//////////////////////////////////////////////////////////////////////////////

CRuleInvokeStmt::CRuleInvokeStmt(CRule *rule, char *iname):
  _first_r(rule),
  _first_rb(NULL),
  _second(NULL)
{
  if (iname)
  	{
  	  _second = new char[strlen(iname)+1];
  	  strcpy(_second, iname);
  	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CRuleInvokeStmt
// Parameters: (CRuleBlock*, char*)
//
// Constructor for rule block invoke.
//         								    
//////////////////////////////////////////////////////////////////////////////

CRuleInvokeStmt::CRuleInvokeStmt(CRuleBlock *rule_block, char *iname):
  _first_rb(rule_block),
  _first_r(NULL),
  _second(NULL)
{
  if (iname)
  	{
  	  _second = new char[strlen(iname)+1];
  	  strcpy(_second, iname);
  	}
  key_number=0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ~CRuleInvokeStmt
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CRuleInvokeStmt::~CRuleInvokeStmt(void)
{
  _first_rb = NULL;
  _first_r = NULL;
  if (_second) { delete [] _second; _second = NULL; }

  // We do not free _first_rb nor _first_r. 
  // The rule will be deleted outside CStmts explicitly. Don't worry about it!
}



CState *CRuleInvokeStmt::ExecDebug(CState *s, DDebugController *db)
{
  int tempNum;
  if (db->really_execute()) {
    if (db->check_for_break(key_number)) {
      if (_second) {
	
	//Mark in environment that this Node is about to have a transformation called on it
	s->IFindDeep(_second)->SetChild(db->setToExec(s->IFindDeep(_second)->Child(), rep_string(), _second));

	//Get Variable Num of _second Node in environment
	tempNum = s->IFindDeep(_second)->Child_Look_Up()->getVariableNum();

	//Make sure that the variableNum in the Node's pattern matches the variable num in is CIdent
	CPattern* tmpPattern = s->IEnv()->findNodeDeep(_second)->GetData()->GetPattern();
	if (tmpPattern)
	  tmpPattern->updateNTVarNum(tempNum);

	//Now that two variableNums have been updated, rebuild s->Store to reflect this
	s = updateDEP(_second, s, s->IEnv()->getData(), 1);
	if (s->CurrentPattern())
	  s = s->CurrentPattern()->buildCurrent(s);
      }
      else {
	db->setToExec(s->Store(), rep_string(), NULL);
      }
      db->doBreak(this, s);
    }

    if (_second) {
      //now that the debugger has returned control back to this method, we can remove the mark on "second" which the debugger used to create an arrow that pointed this node to indicate to the user that a transformation call on this node was about to occur
      s->IFindDeep(_second)->SetChild(db->resetOldVariableNum(s->IFindDeep(_second)->Child(), _second, s));
      
      //Get Variable Num of _second Node in environment
      tempNum = s->IFindDeep(_second)->Child_Look_Up()->getVariableNum();
      
      //Make sure that the variableNum in the Node's pattern matches the variable num in is CIdent
      CPattern* tmpPattern = s->IEnv()->findNodeDeep(_second)->GetData()->GetPattern();
      if (tmpPattern)
	tmpPattern->updateNTVarNum(tempNum);
    }
    else
      db->resetOldVariableNum(s->Store(), NULL, NULL);
      
    db->setToExec(NULL, NULL, NULL);
    db->setTempStmt(this);
    if (db->really_execute()) {
      NonType * old_root = NULL;  
      
      if (s->Store()) {
	old_root = s->Store()->Copy_kTree();
	s->SetRootStore(old_root);
      }
      else
	{
	  cout << "ERROR @ CRuleInvokeStmt : ExecDebug" << endl;
	  cout << "_Store is NULL error" << endl;
	  exit(1);
	}
      
      CIdent *matchNode;
      int is_matchNode_new_one = 0;
      
      // If we are invoking with an identifier, matchNode will have information
      // about the matching node in the query tree. Else (we are invoking with
      // local root of the query tree), matchnode information will correspond
      // to the local root node.
      
      if (_second) 
	{
	  s->IDeleteInvDep(_second);
	  matchNode = s->IFindDeep(_second);
	}
      else 
	{
	  s->CDeleteInvDep();
	  s->CCleanInvDep();
	  matchNode = new CIdent(s->Store(), 1);
	  is_matchNode_new_one = 1;
	}
      
      // Make the local root be the matching node.
      
      s->SetStore(matchNode->Child());
      
      // Flush out the identifier environment 
      s->INew();
      
      // Execute!
      if(_first_rb) 
	{
	  DEPList* tmpDEPList = s->GetInvDep();
	  DEPList* emptyDEPList = new DEPList();
	  s->SetInvDep(emptyDEPList);
	  
	  CIEnvStack * tmpStack = s->IEnv();
	  CIEnvStack * emptyStack = new CIEnvStack();
	  s->SetIEnv(emptyStack);
	  //	  s->INew();
	  //if (is_matchNode_new_one)
	  //  db->setNewKOrig(s);
	  if (_second)
	    db->histPush(old_root->Copy_kTree(), -1, matchNode->Which(), _second, tmpStack, s->CurrentPattern()->getNT());
	  //else
	  //  db->histPush(old_root->Copy_kTree(), -2, 0, NULL, tmpStack, NULL);
	  s = _first_rb->ExecDebug(s, db);
	  if (_second)
	    db->histPop();
	  
	  //	  s->IOld();
	  emptyDEPList = NULL;
	  s->DelInvDep();
	  s->SetInvDep(tmpDEPList);
	  s->SetIEnv(tmpStack);
	  if (emptyStack) { delete emptyStack; emptyStack = NULL; }
	  
	}
      else s = _first_r->ExecDebug(s, db, this);
      
      // Store the success flag in a temporary storage.
      int success = s->ISuccess();
      
      // Restore the identifier environment.
      s->IOld();
      
      // s->Store() points to the transformed query. Make the new information
      // available in the identifier environment.
      
      if (_second) {
	s->IModifyFromHere(_second, 
			   s->Store(), 
			   matchNode->Which(), s->IEnv()->getData());
      }
      
      
      // If the matching node was not the overall root of the tree, 
      // return to the old local root of the tree.
      if (!is_matchNode_new_one) s->SetStore(old_root);
      
      // Build it referring the current pattern in the last match (or match
      // each child) statement. The reconstruction will happen on the subtree
      // of local root of the tree.
      if (_second) 
	{
	  s->SetPatternDeep(_second,NULL);
	  s = updateDEP(_second, s, s->IEnv()->getData(), 1);
	  if (s->CurrentPattern())
	    s = s->CurrentPattern()->buildCurrent(s);
	  else
	    {
	      cout << "Error @ CRuleInvokeStmt : ExecDebug" << endl;
	      cout << "Current pattern is NULL error " << endl;
	      exit(1);
	    }
	}
      
      // Set it success value of rule invoke
      s->SetSuccess(success);

      //s->SetRootStore(s->Store());

      if (is_matchNode_new_one) 
	{ 
	  delete old_root; old_root = NULL; 
	  delete matchNode; matchNode = NULL; 
	}
      db->check_for_condition(s, rep_string(), key_number);
    }
  }
  return s;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters: (CState*)
//
// What to do when executed.
//         								    
//////////////////////////////////////////////////////////////////////////////

CState *CRuleInvokeStmt::Exec(CState *s)
{
  if(_debug_)
	cout << "-- RuleInvokeStmt Begins" << endl;

  NonType * old_root = NULL;  

  if (s->Store())
	old_root = s->Store()->Copy_kTree();
  else
	{
	  cout << "ERROR @ CRuleInvokeStmt : Exec" << endl;
	  cout << "_Store is NULL error" << endl;
	  exit(1);
	}
  
  CIdent *matchNode;
  int is_matchNode_new_one = 0;

  // If we are invoking with an identifier, matchNode will have information
  // about the matching node in the query tree. Else (we are invoking with
  // local root of the query tree), matchnode information will correspond
  // to the local root node.
  
  if (_second) 
	{
	  s->IDeleteInvDep(_second);
	  matchNode = s->IFindDeep(_second);
	}
  else 
	{
	  s->CDeleteInvDep();
	  s->CCleanInvDep();
	  matchNode = new CIdent(s->Store(), 1);
	  is_matchNode_new_one = 1;
	}

  // Make the local root be the matching node.

  s->SetStore(matchNode->Child());

  // Flush out the identifier environment 
  s->INew();

  // Execute!
  if(_first_rb) 
	{
	  DEPList* tmpDEPList = s->GetInvDep();
	  DEPList* emptyDEPList = new DEPList();
	  s->SetInvDep(emptyDEPList);

	  CIEnvStack * tmpStack = s->IEnv();
	  CIEnvStack * emptyStack = new CIEnvStack();
	  s->SetIEnv(emptyStack);
	  //	  s->INew();

	  s = _first_rb->Exec(s);

	  //	  s->IOld();
	  emptyDEPList = NULL;
	  s->DelInvDep();
	  s->SetInvDep(tmpDEPList);
	  s->SetIEnv(tmpStack);
	  if (emptyStack) { delete emptyStack; emptyStack = NULL; }

	}
  else s = _first_r->Exec(s);

  // Store the success flag in a temporary storage.
  int success = s->ISuccess();

  // Restore the identifier environment.
  s->IOld();

  // s->Store() points to the transformed query. Make the new information
  // available in the identifier environment.

  if (_second) 
	s->IModifyFromHere(_second, 
			   s->Store(), 
			   matchNode->Which(), s->IEnv()->getData());


  // If the matching node was not the overall root of the tree, 
  // return to the old local root of the tree.
  if (!is_matchNode_new_one) s->SetStore(old_root);

  // Build it referring the current pattern in the last match (or match
  // each child) statement. The reconstruction will happen on the subtree
  // of local root of the tree.
  if (_second) 
	{
	  s->SetPatternDeep(_second,NULL);
	  s = updateDEP(_second, s, s->IEnv()->getData(), 1);
	  if (s->CurrentPattern())
		s = s->CurrentPattern()->buildCurrent(s);
	  else
		{
		  cout << "Error @ CRuleInvokeStmt : Exec" << endl;
		  cout << "Current pattern is NULL error " << endl;
		  exit(1);
		}
	}

  // Set it success value of rule invoke
  s->SetSuccess(success);
  
  if(_debug_)
	cout << "-- RuleInvokeStmt Ends" << endl;
  
  if (is_matchNode_new_one) 
	{ 
	  delete old_root; old_root = NULL; 
	  delete matchNode; matchNode = NULL; 
	}
  
  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRuleInvokeStmt::rep_string ()
{ 
  if (_second) { 
    if(_first_rb) {
      char *rb_name = _first_rb -> get_name();
      char *res = new char [strlen (_second) + strlen(rb_name) + strlen (" ()") + 1];
      sprintf (res, "%s (%s)", rb_name, _second);
      delete [] rb_name;
      return res;
    }
    else {
      char *the_rule = _first_r -> representation ();
      char *res = new char [strlen (the_rule) + strlen (_second) + strlen (" ()") + 1];
      sprintf (res, "%s (%s)", the_rule, _second);
      delete [] the_rule;
      return res;
    } 
  }
  else {
    if(_first_rb) {
      return _first_rb -> get_name();
    }
    else {
      char *the_rule = _first_r -> representation ();
      char *res = new char [strlen (the_rule) + strlen ("") + 1];
      sprintf (res, "%s", the_rule);
      delete [] the_rule;
      return res;
    } 
  }
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRuleInvokeStmt::representation (int indent)
{
  return rep_string();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController *)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CRuleInvokeStmt::representationWDI (int indent, DDebugController *db)
{
  char* temp = rep_string();
  int currLine = db->getLine();
  int currCol = db->getCol();
  db->updateArray(key_number-1, currLine, currCol, currLine, currCol + strlen(temp));
  return temp;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: number_self
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Uses start parameter to assign key_number data_member, calls number_self 
// on any sub_statements and returns current key-number
//         								    
//////////////////////////////////////////////////////////////////////////////

int CRuleInvokeStmt::number_self (int start)
{
  key_number=start;
  if(_first_rb)
    return (_first_rb -> number_self(start+1));
  else
    return (start + 1);
}

int CRuleInvokeStmt::is_leaf()
  //returns whether current statement is a leaf in the COKO tree
{
  return 1;
}
