//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Joon S. Lee (jl@cs.brown.edu)
//  FILE: PCSkipStmt.C
//  PROJ: COKO
//  DATE: 7/19/98
//
// SYNOPSIS
//  Class definition for CSkipStmt class.
//  
//////////////////////////////////////////////////////////////////////////////

#include <assert.h>
#include "PCState.H"
#include "PCStmt.H"
#include "PCSkipStmt.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CSkipStmt
// Parameters: (void)
//
// Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CSkipStmt::CSkipStmt(void)
{
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: CSkipStmt
// Parameters: (void)
//
// Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

CSkipStmt::~CSkipStmt(void)
{
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: Exec
// Parameters: (CState*)
//
// Do Nothing.
// Set _Success to true.
//
//////////////////////////////////////////////////////////////////////////////

CState* CSkipStmt::Exec(CState *s)
{
  s->SetSuccess(T);
  // set _Success to True 
  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: ExecDebug
// Parameters: (CState*, DDebugController*)
//
// Do Nothing.
// Set _Success to true.
//
//////////////////////////////////////////////////////////////////////////////

CState* CSkipStmt::ExecDebug(CState *s, DDebugController *db)
{
  if (db->really_execute()) {
    s->SetSuccess(T);
    // set _Success to True
  } 
  return s;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: rep_string
// Parameters:	(void)
//
// Returns the string representation of only the current 
// node of the COKO tree
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CSkipStmt::rep_string ()
{
  char *res = new char [strlen ("SKIP") + 1];
  sprintf (res, "SKIP");
  return res; 
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representation
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Returns string representation of COKO tree starting from this statement
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CSkipStmt::representation (int indent) {
  return rep_string();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: representationWDI
// Parameters:	(int, DDebugController *)
//
// ADDED DJA SUMMER 2001
// returns the string representation of this COKO RuleBlock, but
// at the same time updates an array in db that locates sub-strings for
// each key number
//         								    
//////////////////////////////////////////////////////////////////////////////

char* CSkipStmt::representationWDI (int indent, DDebugController *db)
{
  char* temp = rep_string();
  db->setCol(db->getCol() + strlen(temp));
  return temp;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: number_self
// Parameters:	(int)
//
// ADDED DJA SUMMER 2001
// Uses start parameter to assign key_number data_member, calls number_self 
// on any sub_statements and returns current key-number
//         								    
//////////////////////////////////////////////////////////////////////////////

int CSkipStmt::number_self (int start)
{
  key_number=0; //SKIP not a breakable statement
  return start;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: output_self
// Parameters:	(DDebugController *)
//
// ADDED DJA SUMMER 2001
// Outputs current statement in list format (for the debugger) if it is a 
// breakable statement; otherwise just calls output_self on any sub-statements
//         								    
//////////////////////////////////////////////////////////////////////////////

void CSkipStmt::output_self (DDebugController *db)
{
  //SKIP is not a breakable statement
  ;
}

int CSkipStmt::is_leaf()
  //returns whether current statement is a leaf in the COKO tree
{
  return 1;
}
