/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)					 	 
// FILE: ENEnvironment.C
// PROJ: KOLA Parser, AQUA Translator
// DATE: 5/95
//
//////////////////////////////////////////////////////////////////////////////

#include "ENEnvironment.H"


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Environment
// Parameters:	()							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

Environment::Environment ()
{
 //  fst = new EnvEntry ("NIL", new FID (), new TAny ());
  //lst = fst;
	 fst = NULL;
	 lst = NULL;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Environment
// Parameters:	(Environment& other)							  
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

Environment::Environment (Environment& other)
{


   if (other . Top () == NULL)
       fst = NULL;
   else
       fst = new EnvEntry (*(other . Top ()));

}   


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~Environment
// Parameters:	()							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

Environment::~Environment ()
{

  if (fst) { delete fst; fst = NULL;}
    
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  show
// Parameters:	()							  
//
//  Return textual representation of this.
//         								    
//////////////////////////////////////////////////////////////////////////////

char *Environment::show ()
{

    if (fst != NULL)
      return fst -> show ();
    else {
      char *res = new char [1];
      sprintf (res, "");
      return res;
    }
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  GetFunc ()
// Parameters:	(char *var)							  
//
// Return function that gets var by looking up in environment
//         								    
//////////////////////////////////////////////////////////////////////////////

Function *Environment::GetFunc (char *var)
{

  if (fst == NULL) 
      return NULL;
  else    
      return fst -> GetFunc (var);
  
}




//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  GetType ()
// Parameters:	(char *var)							  
//
// Return type that gets var by looking up in environment
//         								    
//////////////////////////////////////////////////////////////////////////////

TObj *Environment::GetType (char *var)
{
    
  if (fst == NULL) 
      return NULL;
  else    
      return fst -> GetType (var);
  
}




//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  add_entry ()
// Parameters:	(char *var)							  
//
// Adds (var, _2) to top of environment, and adds o _1 to every other entry
//         								    
//////////////////////////////////////////////////////////////////////////////

Environment *Environment::add_entry (char *var, TObj *t)
{

  if (fst == NULL) 
      fst = new EnvEntry (var, new FID (), t);
  else {      
      fst -> Add_one ();
      EnvEntry *nextop = new EnvEntry (var, new F2 (), t, fst);
      fst = nextop;
      
  };

  return this;

    
//   if (fst == NULL) {
//       fst = new EnvEntry (var, new FID (), t);
//       lst = fst;
//   } else {      
//       fst -> Add_one ();
//       EnvEntry *nextop = new EnvEntry (var, new F2 (), t);
//       lst -> Next (nextop);
//       lst = nextop;   
//   };
// 
//   return this;
        
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  add_pair_entry ()
// Parameters:	(char *var1, char *var2)							  
//
// Adds (var1, _1 o _2), (var2, _2 o _2) to top of environment, and adds o _1 to every other entry
//         								    
//////////////////////////////////////////////////////////////////////////////

Environment *Environment::add_pair_entry (char *var1, char *var2, TObj *t1, TObj *t2)
{

  if (fst == NULL) {
      EnvEntry *one = new EnvEntry (var1, new F1 (), t1);
      fst = new EnvEntry (var2, new F2 (), t2, one);
      lst = one;
  } else {
      fst -> Add_one ();
      EnvEntry *one = new EnvEntry (var1, new FCompose (new F1 (), new F2 ()), t1, fst);
      fst = new EnvEntry (var2, new FCompose (new F2 (), new F2 ()), t2, one);      
  };

  return this;

    
//   if (fst == NULL) {      
//       EnvEntry *two = new EnvEntry (var2, new F2 (), t2);
//       EnvEntry *one = new EnvEntry (var1, new F1 (), t1, two);
//       lst -> Next (one);
//       lst = two;
//   } else {
//       fst -> Add_one ();
//       EnvEntry *two = new EnvEntry (var2, new FCompose (new F2 (), new F2 ()), t2);
//       EnvEntry *one = new EnvEntry (var1, new FCompose (new F1 (), new F2 ()), t1, two);
//       lst -> Next (one);
//       lst = two;
//   };  
// 
//   return this;
        
}




//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  add_manual_entry ()
// Parameters:	(char *var, TObj *typ)							  
//
// Manually adds (var, ID, typ) to environment (used for Schema Manager)
//         								    
//////////////////////////////////////////////////////////////////////////////

Environment *Environment::add_manual_entry (char *var, TObj *t)
{

  if (fst == NULL) {
      fst = new EnvEntry (var, new FID (), t);
      lst = fst;      
  } else {
      EnvEntry *nextop = new EnvEntry (var, new FID (), t, fst);
      fst = nextop;
  }

  return this;
        
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  add_manual_entry ()
// Parameters:	(char *var, Function *f, TObj *typ)							  
//
// Manually adds (var, f, typ) to environment (used for Label Manager)
//         								    
//////////////////////////////////////////////////////////////////////////////

Environment *Environment::add_manual_entry (char *var, Function *f, TObj *t)
{

  if (fst == NULL) {
      fst = new EnvEntry (var, f, t);
      lst = fst;      
  } else {
      EnvEntry *nextop = new EnvEntry (var, f, t, fst);
      fst = nextop;
  }

  return this;
        
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reversed ()
// Parameters:	()
//
// Return chain of copies of EnvEntries in chain buit in reversed order
//         								    
//////////////////////////////////////////////////////////////////////////////

EnvEntry *Environment::reversed ()
{

  if (fst == NULL) return NULL;
  else return fst -> reversed ();
        
}



 
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  insert ()
// Parameters:	(EnvEntry *ent)
//
// Insert ent into environment
//         								    
//////////////////////////////////////////////////////////////////////////////

Environment *Environment::insert (EnvEntry *ent)
{
    
  if (fst == NULL) {
      fst = ent;
      lst = ent;
  } else {
      ent -> Next (fst);
      fst = ent;
  };

  return this;
        
}




//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  buildtup_unnest 
// Parameters:	(char *fn)
//
// Builds a tuple from the environment, but replacing label fn with the function f
// Used to translate unnest
//
//////////////////////////////////////////////////////////////////////////////

Function *Environment::buildtup_unnest (char *fn)
{

  if (fst == NULL)
      return new FID ();
  else {
      
    Environment *rev = (new Environment ()) -> insert (reversed ());

    EnvEntry *p = rev -> Top ();
    Function *res = NULL;

    while (p != NULL) {
	if (res == NULL) {	  
            if (strcmp (p -> Variable (), fn) == 0)
		res = new F2 ();
	    else
		res = new FCompose ((p -> Funct ()) -> Copy (), new F1 ());
	} else {
            if (strcmp (p -> Variable (), fn) == 0)
		res = new FPlus (res, new F2 ());
	    else
		res = new FPlus (res, new FCompose ((p -> Funct ()) -> Copy (), new F1 ()));
        };

	p = p -> Next ();
	
    };
	
    delete rev;
    return res;
  }    
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  buildtup_nest 
// Parameters:	(char *fn)
//
// Builds a tuple from the environment, but replacing label fn with the function f
// Used to translate nest
//
//////////////////////////////////////////////////////////////////////////////

Function *Environment::buildtup_nest (char *fn)
{

  if (fst == NULL)
      return new FID ();
  else {
      
    Environment *rev = (new Environment ()) -> insert (reversed ());

    EnvEntry *p = rev -> Top ();
    Function *res = NULL;
    PConst *pr = new PConst (new BPrimitive (T));
    
    while (p != NULL) {
	if (res == NULL) {	  
            if (strcmp (p -> Variable (), fn) == 0)
		res = new FIterate (pr -> Copy (), (p -> Funct ()) -> Copy ());
	    else
		res = new FCompose ((p -> Funct ()) -> Copy (), new FChoose ());
	} else {
            if (strcmp (p -> Variable (), fn) == 0)
		res = new FPlus (res, new FIterate (pr -> Copy (), (p -> Funct ()) -> Copy ()));
	    else
		res = new FPlus (res, new FCompose ((p -> Funct ()) -> Copy (), new FChoose ()));
        };

	p = p -> Next ();
	
    };

    delete pr;
    delete rev;
    return res;
  }    
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  buildtup_without 
// Parameters:	(char *fn)
//
// Builds a tuple from the environment, but without label fn
// Used to translate nest
//
//////////////////////////////////////////////////////////////////////////////

Function *Environment::buildtup_without (char *fn)
{

  if (fst == NULL)
      return new FID ();
  else {
      
    Environment *rev = (new Environment ()) -> insert (reversed ());

    EnvEntry *p = rev -> Top ();

    Function *res = NULL;

    while (p != NULL) {
	if (res == NULL) {
            if (strcmp (p -> Variable (), fn) != 0)
		res = (p -> Funct ()) -> Copy ();
	} else {
            if (strcmp (p -> Variable (), fn) != 0)
		res = new FPlus (res, (p -> Funct ()) -> Copy ());
	};
	p = p -> Next ();
	
    };
	
    delete rev;
    return res;
  }    
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  buildtuptype_unnest
// Parameters:	(char *fn)
//
// Builds a tuple type from the environment, but replacing the type for label fn (which has type SET [t])
// with the type t.
// Used to translate unnest
//
//////////////////////////////////////////////////////////////////////////////

TObj *Environment::buildtuptype_unnest (char *fn)
{

  if (fst == NULL)
      return NULL;
  else {

    EnvEntry *pm = reversed ();
    EnvEntry *p = pm;
    Environment *en;
    TTup *res = new TTup (new Environment ());

    while (p != NULL) {
       if (strcmp (p -> Variable (), fn) == 0)
	   en = (res -> Env ()) ->
		  	add_manual_entry (p -> Variable (), (p -> Funct ()) -> Copy (), (((p -> Type ()) -> asSet ()) -> One ()) -> Copy ());
       else
	   en = (res -> Env ()) ->
			add_manual_entry (p -> Variable (), (p -> Funct ()) -> Copy (), (p -> Type ()) -> Copy ()); 

       p = p -> Next ();
	
    };

    delete pm;
    return res;
  }    
}




//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  buildtuptype_nest
// Parameters:	(char *fn)
//
// Builds a tuple type from the environment, but replacing the type for label fn (which has type SET [t])
// with the type t.
// Used to translate nest
//
//////////////////////////////////////////////////////////////////////////////

TObj *Environment::buildtuptype_nest (char *fn)
{

  if (fst == NULL)
      return NULL;
  else {
      
    EnvEntry *p = reversed ();
    EnvEntry *pm = p;

    TTup *res = new TTup (new Environment ());
    Environment *en;

    while (p != NULL) {
        if (strcmp (p -> Variable (), fn) == 0)
	     en = (res -> Env ()) ->
		  	add_manual_entry (p -> Variable (), (p -> Funct ()) -> Copy (), new TSet ((p -> Type ()) -> Copy ()));
 	else
	     en = (res -> Env ()) ->
		  	add_manual_entry (p -> Variable (), (p -> Funct ()) -> Copy (), (p -> Type ()) -> Copy ());

	p = p -> Next ();
	
    };

    delete pm;
    return res;
  }    
}










//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sameas
// Parameters:	(Environment *other)
//
// Return true if this environment has same fields/types as other
//
//////////////////////////////////////////////////////////////////////////////

int Environment::sameas (Environment *other)
{

    int res = T;
    EnvEntry *tptr = fst;
    while (res == T && tptr != NULL) {
	TObj *t = tptr -> Type ();
	TObj *t2 = other -> GetType (tptr -> Variable ());
	if (t == NULL || t2 == NULL || ! (t -> sameas (t2)))
	    res = F;
        tptr = tptr -> Next ();
    };
    return res;
    
}










