/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: KOFUnnest.C
// PROJ: KOLA Parser
// DATE: 5/97
//
//////////////////////////////////////////////////////////////////////////////

#include "KOFUnnest.H"
//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FUnnest
// Parameters:	(Function* f1, Function *f2)							  
//
//  Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////


FUnnest::FUnnest (Function *f1, Function *f2, TFunc * ftype)
                                 : Function (ftype, FUN_UNNEST,"UNNEST")
{
  one = f1;
  two = f2;
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  FUnnest
// Parameters:	(FUnnest& other)							  
//
//  Copy Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

FUnnest::FUnnest (const FUnnest& other) : Function ((Function &) other)
{
  one = (other . One ()) -> Copy ();
  two = (other . Two ()) -> Copy ();
  variableNum = other.getVariableNum();
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~FUnnest
// Parameters:	()							  
//
//  Destructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

FUnnest::~FUnnest ()
{
  delete one;
  delete two;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy ()
// Parameters:	()							  
//
//  Virtual Copy Constructor 
//         								    
//////////////////////////////////////////////////////////////////////////////

Function *FUnnest::Copy () const
{
  FUnnest* temp = new FUnnest (*this);
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  deepeq
// Parameters:  ()                                                        
//
// Value-based equality.
// this is computed recursively.
// Return true if two trees have identical structures, otherwise return false.
// No unification occurs here.
//                                                                          
//////////////////////////////////////////////////////////////////////////////

int FUnnest::deepeq (NonType * other) 
{
  NameUnion* name_ = other->Myname();
  if ((name_->tag == FNAME ) && (name_->fn == FUN_UNNEST))
  {
    delete name_;
    return ((one->deepeq(((FUnnest*)other)->One())) 
         && (two->deepeq(((FUnnest*)other)->Two())));
  }
  else
    {
      delete name_;
      return F;
    }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* FUnnest::Myname()
{
	NameUnion* a = new NameUnion(FNAME, FUN_UNNEST);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  nextChild
// Parameters:  ()
//
// Returns "currChild++"th child.
// When currChild is equal to the number of children
// (when current child is the last child), it returns NULL.
//
//////////////////////////////////////////////////////////////////////////////

NonType* FUnnest::nextChild()
{
  if (currChild == 0)
	{
	  currChild++;
	  return one;
	}
  else if (currChild == 1)
	{
	  currChild++;
	  return two;
	}
  else return NULL;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//
/////////////////////////////////////////////////////////////////////////////

// OResult* FUnnest::reduce (OResult* a)
// {
//   OResVAR* v1 = new OResVAR ();
//   OResVAR* v2 = new OResVAR ();
  
//   OResSET *res = 
//     new OResSET (new OResPAIR (one -> reduce (v1 -> Copy ()), v2 -> Copy ()),
// 		 new BResCOMP (new BResINF (v1 -> Copy (), IN, a -> Copy ()),
// 			       SAND,
// 			       new BResINF (v2 -> Copy (), IN, two -> reduce (v1 -> Copy ()))));
			       
//   delete v1;
//   delete v2;
//   delete a;
//   return res;
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "FFO: UNNEST (" + one -> representation () + two -> representation () + ")"
//         								    
//////////////////////////////////////////////////////////////////////////////

char* FUnnest::representation ()
{
  char *orr = one -> representation ();
  char *tr = two -> representation ();
  char *res = new char [strlen (orr) + strlen (tr) + strlen ("UNNEST (, )") + 1];
  sprintf (res, "UNNEST (%s, %s)", orr, tr);
  delete [] orr;    
  delete [] tr;    
  return res;
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* FUnnest::transform(CState* s)
{
  Function* first = (Function*) One() -> transform(s);
  Function* second = (Function*) Two() -> transform(s);
  FUnnest* temp = new FUnnest(first,second,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transformDeep
// Parameters:  (CState*, CIEnvStackDataType*)                                                    
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//  The difference between this method and transform method is a search
//  method used in VAR classes. (Take a look at transform and transformDeep
//  methods in BVar, FVar, PVar, or OVar.)
//  Transform method searches the top frame only, 
//  while TransformDeep methods searches entire stack starting from 
//  a given point. 
//
//////////////////////////////////////////////////////////////////////////////

NonType* FUnnest::transformDeep(CState* s, CIEnvStackDataType* startingPt)
{
  Function* first = (Function*) One() -> transformDeep(s, startingPt);
  Function* second = (Function*) Two() -> transformDeep(s, startingPt);
  FUnnest* temp = new FUnnest(first,second,Type());
  temp->setVariableNum(variableNum);
  return temp;
    
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here. 
//
// For example, foo: g o g --> g o g is a legal rule.
//                  -------   -------
//                    |         |_\ 
//                    |_\         / building part
//                      / matching part 
//
// Assume that we apply this rule to a kola tree of the form "ID o ID."
// When unification occurs in a matching part, first we store first g and ID pair 
// into the env stack. Then the next time we face another g (second one), 
// we search the stack for the matching pair with variable name g and simply check 
// if the matching pair in the stack has same structure as the matching pair we have.
// In this case, second pair is (g, ID) as well and the rule gets fired successfully.
//
// Now assume that we apply the same rule to a kola tree of the form "ID o (ID o ID)."
// In this case, the rule fails since the second pair is (g, ID o ID),
// while the first pair is (g, ID). 
//
// Note that even in a case of failure, the rule itself is legal.
//
// "Duplication of a variable is legal in a rule section."
//
//////////////////////////////////////////////////////////////////////////////

int FUnnest::compares(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();
  
  if ((tmp_name->tag == FNAME) && (tmp_name->fn == FUN_UNNEST))
	{
	  // do it for first child.
	  if (one->compares(((FUnnest*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one))
			// child (one) is a variable.
			{
			  if ((a->IFind(((FVar*)one)->Name())) == NULL)
				{
				  // child is a new variable.
				  a->IInsert(((FVar*)one)->Name(), new CIdent((FUnnest*)nt,((FUnnest*)nt)->One(),1));
				  // Push entry into the stack.
				}
			  else 
				{
				  // Child is a variable but not new one.
				  // In this case, I need to make sure that what's
				  // already in the stack and what I have now is the same.
				  if (!(((a->IFind(((FVar*)one)->Name()))->Child_Look_Up())->deepeq(((FUnnest*)nt)->One())))
					return_val++;
				  // match failed.
				  // else do none.
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++;
	  // compare failed
	  
	  // do it for second child.
	  if (two->compares(((FUnnest*)nt)->Two(), a))
		{
		  if (isChildVar(two))
			{
			  if ((a->IFind(((FVar*)two)->Name())) == NULL)
				{
				  a->IInsert(((FVar*)two)->Name(), new CIdent((FUnnest*)nt, ((FUnnest*)nt)->Two(), 2));
				}
			  else 
				{
				  if (!(((a->IFind(((FVar*)two)->Name()))->Child_Look_Up())->deepeq(((FUnnest*)nt)->Two())))
					return_val++;
				}
			}
		}
	  else return_val++;
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0.
  // In this case, return false(0).
  // Otherwise, return true(1).
  
  delete tmp_name;
  
  if (return_val)
	return F;
  else
	return T;
  
} 

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: comparesSTM
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for statement matching only.
// Statement matching is different from rule matching.
// No duplicated variable entry is allowed here. 
//
// For example, 
//             match f ! _O,
//                   f =  g o g  is a illegal statement (equation).
//                       -------   
//                         |       
//                         |_\     
//                           / matching part 
//
// Assume that we have a query tree of the form "ID o ID ! 3."
// Variable f gets bounded to "ID o ID" and we have to match "ID o ID" to "g o g."
// Matching "ID o ID" to "g o g" is perfectly legal in a rule section.
// However, it is illegal in statement section of the program.
// Remember that every variable on the right hand side of an equation must be a fresh one.
// However, after we match first g to ID, the second g is no longer fresh variable.
//
// "No duplication of a variable is legal in a statement section."
// 
//////////////////////////////////////////////////////////////////////////////

int FUnnest::comparesSTM(NonType* nt, CState* a)
{
  int return_val = 0;
  NameUnion* tmp_name = nt->Myname();
  
  if ((tmp_name->tag == FNAME) && (tmp_name->fn == FUN_UNNEST))
	{
	  // do it for first child.
	  if (one->comparesSTM(((FUnnest*)nt)->One(), a))
		// compare succeeded.
		{
		  if (isChildVar(one))
			// child (one) is a variable.
			{
			  if ((a->IFind(((FVar*)one)->Name())) == NULL)
				{
				  // child is a new variable.
				  a->IInsert(((FVar*)one)->Name(), new CIdent((FUnnest*)nt,((FUnnest*)nt)->One(),1));
				  // Push entry into the stack.
				}
			  else 
				{
					return_val++;
					cout << "Error : Variable " << ((FVar*)one)->Name() << " is used twice. " << endl;
				}
			}
		  // else do none.
		  // no child is variable.
		}
	  else return_val++;
	  // compare failed
	  
	  // do it for second child.
	  if (two->comparesSTM(((FUnnest*)nt)->Two(), a))
		{
		  if (isChildVar(two))
			{
			  if ((a->IFind(((FVar*)two)->Name())) == NULL)
				{
				  a->IInsert(((FVar*)two)->Name(), new CIdent((FUnnest*)nt, ((FUnnest*)nt)->Two(), 2));
				}
			  else 
				{
				  return_val++;
				  cout << "Error : Variable " << ((FVar*)two)->Name() << " is used twice. " << endl;
				}
			}
		}
	  else return_val++;
	}
  else return_val++;
  // When one of it's child is a variable and faild match, return_val
  // is set to some number bigger than 0.
  // In this case, return false(0).
  // Otherwise, return true(1).
  
  delete tmp_name;
 
  if (return_val)
	return F;
  else
	return T;

} 

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  setDEPnInvDEP
// Parameters:  (char*, CState*)
//
// set dependency list (ancestor list) and inverse dependency list (descendant list)
//
//////////////////////////////////////////////////////////////////////////////

void FUnnest::setDEPnInvDEP(char* vn, CState* s)
{
  one->setDEPnInvDEP(vn,s);
  two->setDEPnInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  undoSetInvDEP
// Parameters:  (char*, CState*)
//
// undo changes made on descendant list (inverse dependency list).
// undo for dependency list (ancestor list) is not necessary, since 
// any changes on dependency list is undone by poping up
// top frame from the environment stack.
//
//////////////////////////////////////////////////////////////////////////////

void FUnnest::undoSetInvDEP(char* vn, CState* s)
{
  one->undoSetInvDEP(vn,s);
  two->undoSetInvDEP(vn,s);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void FUnnest::replaceChild(NonType* child, int which_child)
{
  if ( which_child == 1 )
	SetOne(child);
  else if ( which_child == 2 )
	SetTwo(child);
}
 
#ifdef _OSTORE_
//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: exec
//  Parameters:(OSObject*)
//
//  Implements the unnest function former.
//
//////////////////////////////////////////////////////////////////////////////

OSObject* FUnnest::exec(OSObject* x)
{
  return new UnnestSieve(One(),Two(),((Iterator*)x));
}

#endif // _OSTORE_
  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* FUnnest::prolog ()
{
  char *orr = one -> prolog ();
  char *tr = two -> prolog ();
  char *res = new char [strlen (orr) + strlen (tr) + strlen ("funnest(, )") + 1];
  sprintf (res, "funnest(%s, %s)", orr, tr);
  delete [] orr;    
  delete [] tr;    
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref FUnnest::sp_prolog ()
{
  SP_term_ref or = one -> sp_prolog ();
  SP_term_ref tr = two -> sp_prolog ();
  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("funnest"),2,or,tr);
  return res;
}
#endif // _PROLOG_

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	(koObject *o)
//
// Return the OQL string equivalent of invoking this on o
//
/////////////////////////////////////////////////////////////////////////////

char* FUnnest::oql (koObject *o)
{

    OVar *x   = new OVar (0); 
    OVar *y   = new OVar (0); 
    OPair *op = new OPair (x, y);

    char *sel  = (one -> inv (op)) -> oql ();
    char *fro1 = o -> oql ();
    char *fro2 = (two -> inv (x)) -> oql ();
    
    if (strncmp (sel, "ERROR", 5) == 0) {
	  delete [] fro1;
	  delete [] fro2;
	  delete op;       // deletes x and y
	  return sel;
    } else if (strncmp (fro1, "ERROR", 5) == 0) {
	  delete [] sel;
	  delete [] fro2;
	  delete op;       // deletes x and y
	  return fro1;
    } else if (strncmp (fro2, "ERROR", 5) == 0) {
	  delete [] sel;
	  delete [] fro1;
	  delete op;       // deletes x and y
	  return fro2;
    } else {
      char *res = new char [strlen ("(select  from  ,  )") + 
						    strlen (sel) + strlen (x -> Name ()) + strlen (fro1) + strlen (y -> Name ()) + strlen (fro2) + 1];
      sprintf (res, "(select %s from %s %s, %s %s)", sel, fro1, x -> Name (), fro2, y -> Name ());
      delete [] sel;
      delete [] fro1;
      delete [] fro2;
	  delete op;       // deletes x and y
	  return res;
    }
}
