/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)
//       Joon S. Lee (jl@cs.brown.edu)
// FILE: COOVar.C
// PROJ: COKO
// DATE: 4/96
//
//////////////////////////////////////////////////////////////////////////////

#include "KOOVar.H"
int OVar::cnt = 0;

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OVar
// Parameters:	(char* n)							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OVar::OVar  (char* s, TObj * t) : koObject (t, s)
{
  if (s == NULL) val = NULL;
  else {
    val = new char [strlen(s) + 1];
    sprintf (val, "%s", s);
  }
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OVar
// Parameters:	(OVar& other)							  
//
//  Copy Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OVar::OVar (const OVar& other) : koObject ((koObject &) other)
{
  if (other.val == NULL) val = NULL;
  else {
    val = new char [strlen(other.val) + 1];
    sprintf (val, "%s", other.val);
  }
  variableNum = other.getVariableNum();
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  OVar
// Parameters:	(int i)							  
//
//  Constructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OVar::OVar (int i, TObj * t) : koObject (t, NULL)
{
  val = new char [MAXINTLENGTH + 1];
  sprintf (val, "x%d", cnt);
  namestring = new char [strlen (val) + 1];
  sprintf (namestring, "%s", val);
  cnt++;
  variableNum = 0;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~OVar
// Parameters:	()							  
//
//  Destructor.
//         								    
//////////////////////////////////////////////////////////////////////////////

OVar::~OVar ()
{
 if (val) {delete [] val; val = NULL;}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  reduce
// Parameters:	()							  
//
/////////////////////////////////////////////////////////////////////////////

// OResult* OVar::reduce ()
// {
//   return new OResID ((char *) Name());
// }

// //////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  "OVar: " + name
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OVar::representation ()
{
  //Commented out by DJA: SUMMER 2002 - the O: was ugly
  //char *res = new char [strlen (val) + strlen ("(O: )") + 1];
  //sprintf (res, "(O: %s)", val);

  //Added by DJA: SUMMER 2002
  char* res;
  if ((strlen(val) > 1) && (val[0] == '_') && (val[1] == 'O')) {
    res = new char [strlen ("_O") + 1];
    sprintf (res, "_O");
  }
  else {
    res = new char [strlen (val) + 1];
    sprintf (res, "%s", val);
  }
  return res;


}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Copy
// Parameters:	()							  
//
//  return copy of this as koObject*
//         								    
//////////////////////////////////////////////////////////////////////////////

koObject* OVar::Copy () const
{
  OVar* temp = new OVar (*this);
  temp->setVariableNum(variableNum);
  return temp;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Myname
// Parameters:	()							  
//
//  Returns a union type, NameUnion. 
//  Since NonType has 5 different sub-classes, Function, Predicate, Object, 
//  Bool and Index, and those 4 different sub-classes use different 
//  enum type for their own sub-classes' names, Myname methods needs 
//  a union type of 5 different enum types.x
//  Since Myname is defined virtually in KONonType, one can just call this 
//  methods with NonType* to identify the specific instance of Kola object.
//         								    
//////////////////////////////////////////////////////////////////////////////

NameUnion* OVar::Myname()
{
	NameUnion* a = new NameUnion(ONAME, OBJ_VAR);
	return a;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transform
// Parameters:	(CState*)							  
//
//  if there is a entry for this variable name in the stack
//  return the corresponding Nontype else return NULL
//         								    
//////////////////////////////////////////////////////////////////////////////

NonType* OVar::transform(CState* a)
{
  if ((a->IFind(val))!= NULL)
  {
     a->SetSuccess(T);
     return ((koObject*)((a->IFind(val))->Child()));
  }
  else
  {
    a->SetSuccess(F);
    return NULL;
  }
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: partial_transform
// Parameters:	(CState*)							  
//
// similar to transform except one thing.
// If no match is found for variable,
// do not report it as an error. Instead,
// just return variable itself.
//         								    
//////////////////////////////////////////////////////////////////////////////

NonType* OVar::partial_transform(CState* a)
{
  if ((a->IFind(val)) != NULL)
	{
	  a->SetSuccess(T);
	  return (((koObject*)((a->IFind(val))->Child())));
	}
  else
	{
	  a->SetSuccess(T);
	  return Copy();
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  transformDeep
// Parameters:	(CState*, CIEnvStackDataType*)							  
//
//  Return coresponding NonType object.
//  CState is only used in VAR classes.
//  No other classes need CState.
//  However, CState is kept for making transform methods uniformly. 
//  The difference between this method and transform method is a search
//  method used in VAR classes. (Take a look at transform and transformDeep
//  methods in BVar, FVar, PVar, or OVar.
//
//////////////////////////////////////////////////////////////////////////////

NonType* OVar::transformDeep(CState* a, CIEnvStackDataType* startingPt)
{
  if ((a->IFindFromHere(val, startingPt)) != NULL)
  {
     a->SetSuccess(T);
     return ((koObject*)((a->IFindFromHere(val, startingPt))->Child()));
  }
  else
  {
    a->SetSuccess(F);
    return NULL;
  }
}

//////////////////////////////////////////////////////////////////////////////
//
//  Function Name: compares
//  Parameters:(NonType*, CState*)
//
// Checks for match between two NonType trees (query tree and pattern tree).
// Stores a variable and CIdent pair onto the stack.
// This method is for rule matching only.
// Rule matching is different from statement matching.
// Duplicated variable entry is allowed here. 
//
//////////////////////////////////////////////////////////////////////////////

int OVar::compares(NonType* nt, CState* a)
{
  // Please check: Is this ok?

  NameUnion* tmp_name =  nt->Myname();
  //  if ((tmp_name->tag == ONAME) && (tmp_name->on >= OBJ_ANY) && (tmp_name->on <= OBJ_UNI))
  if ((tmp_name->tag == ONAME) && (tmp_name->on != OBJ_VAR) && (tmp_name->on != OBJ_DC))
	{
	  delete tmp_name;
	  return T;
	}
  else
	{
	  delete tmp_name;
	  return F;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name: directStore
// Parameters:	(CState*)							  
//
//  For variables, I only need to check whether the corresponding
//  Nontype Class is as same type as current NonType.
//
//////////////////////////////////////////////////////////////////////////////

CState* OVar::directStore(CState* a)
{
  if (a->Store() == NULL)
        {
          cout << "EOR -------- EOR ---- Store is NULL :: OVar" << endl;
          return NULL;
        }

  NameUnion* tmp_name =  (a->Store())->Myname();

  //  if ((tmp_name->tag == ONAME) && (tmp_name->on >= OBJ_ANY) && (tmp_name->on <= OBJ_UNI))
  if ((tmp_name->tag == ONAME) && (tmp_name->on != OBJ_VAR) && (tmp_name->on != OBJ_DC))
	{

	  if (((a->IFind(val))) == NULL)
		{
		  a->IInsert(val, new CIdent( NULL, a->Store(), 0));
		  a->SetSuccess(T);
		}
	  else if (((a->IFind(val))) != NULL)
		{
		  if (!((a->IFind(val))->Child_Look_Up())->deepeq (a->Store()))
			a->SetSuccess(F);
		  else
			a->SetSuccess(T);
		}
	}
  else a->SetSuccess(F);
 
 delete tmp_name;
 return a;
}      

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  setInvDEP
// Parameters:  (char*, CState*)
//
// set dependency list and inverse dependency list
//
//////////////////////////////////////////////////////////////////////////////

void OVar::setDEPnInvDEP(char* vn, CState* s)
{
  if (vn == NULL)
	  s->CInsertInvDep(val);
  else
	{
	  s->IInsertInvDep(vn, val);
	  s->IInsertDep(val, vn);
	}
}

//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  undoSetInvDEP
// Parameters:  (char*, CState*)
//
// undo changes made on inverse dependency list.
// undo for dependency list is not necessary, since 
// any changes on dependency list is undone by poping up
// top frame from the environment stack.
//
//////////////////////////////////////////////////////////////////////////////

void OVar::undoSetInvDEP(char* vn, CState* s)
{
  s->IDeleteInvDep(val);
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  replaceChild
// Parameters:	(NonType*, int)
//
// Replace a child of given tree.  
// which_child specifies which child to replace.
// For example, when which_child is one, we replace the first child of 
// given tree.
// 
//////////////////////////////////////////////////////////////////////////////

void OVar::replaceChild(NonType* child, int which_child)
{
}

  
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  prolog
// Parameters:	()
//
// return string representation in prolog term
//
//////////////////////////////////////////////////////////////////////////////

char* OVar::prolog ()
{
  char *res = new char [strlen (val) + strlen ("V") + 1];
  sprintf (res, "V%s", val);
  return res;
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref OVar::sp_prolog ()
{
  cout << "Error @OVar::sp_prlog" << endl;
  cout << "query input should not have variable inside error!" << endl;
  return NULL;
}
#endif // _PROLOG_


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  initstats
// Parameters:  ()                                                        
//
//  Initialize static variables
//                                                                          
//////////////////////////////////////////////////////////////////////////////

void initstats ()
{
    OVar::cnt = 0;
}



//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  sql
// Parameters:	()
//
//  Return the SQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OVar::sql ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, val);
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  oql
// Parameters:	()
//
//  Return the OQL string equivalent of this object
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OVar::oql ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, val);
  return res;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  Val
// Parameters:	()
//
// Selector - return a copy of val
//         								    
//////////////////////////////////////////////////////////////////////////////

char* OVar::Val ()
{
  char *res = new char [strlen (val) + 1];
  sprintf (res, "%s", val);
  return res;
}
