
/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc)
//        Xiaoyu Wang (wangxy)
//  FILE: KOTDollar.H 
//  PROJ: COKO, KOLA
//  DATE: 9/00		                                                    
//
// SYNOPSIS
//  tidla tagexp.  Tag is the tag of this tagexp.
//  which can be either (@ "ID") or (% "ID").
//                       |_          |_
//                         Attr        Ele
//
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_TDOLLAR_HEADER
#define KOLA_TDOLLAR_HEADER 

#include "KGlobals.H"
#include "KOTagExp.H"
#include "KONameUnion.H"
#include "KONonType.H"

#include "PCState.H"
#include <string.h>

class  TDollar: public TagExp
{
public:
  TDollar (TagExp* tg);
  // Constructor.

  int compares(NonType* nt, CState* a);
  // Checks for match between two NonType trees (query tree and pattern tree).
  // Stores a variable and CIdent pair onto the stack.
  // This method is for rule matching only.
  // Rule matching is different from statement matching.
  // Duplicated variable entry is allowed here. 
 
  NonType* transform(CState* s);
  //  Return coresponding Nontype object.
  //  CState is only used in VAR classes.
  //  No other classes need CState.

  void setDEPnInvDEP(char *vn, CState *s); 
  // set dependency list and inverse dependency list

  void undoSetInvDEP(char*, CState*); 
  // undo changes made on inverse dependency list.
  // undo for dependency list is not necessary, since 
  // any changes on dependency list is undone by poping up
  // top frame from the environment stack.

  TDollar (const TDollar& other);
  // Copy Constructor.
  
  ~TDollar ();
  // Destructor.

  TagExp* Copy () const;
  // Virtual copy constructor.
    
  int deepeq(NonType * other);
  // Value-based equality

  NameUnion* Myname();
  // returns myname wrapped inside of a class NameUnion.

  int numChildren() const { return 1; }
  //  Number of arguments.
 
  NonType* Transform(CState *);
 
  int compareSTM(NonType*, CState* s);

  const  char* Name() const { return namestring;}

  TagExp* One() const { return one; }

  void setOne(NonType* te) { delete one; one = (TagExp*)te; }  
 
  void replaceChild( NonType* c, int n );
 
  //  OResult* reduce ();
  //  Return as_string (val)	
  
#ifdef _PROLOG_
  SP_term_ref sp_prolog ();
    // convert kola tree into prolog term and return it.
#endif // _PROLOG_ 

  char* prolog ();
  // return string representation in prolog term

  char* representation ();
  //  Return "DOLLAR" + Tag as a string

  void initCurrChild() { currChild = 0;}
  //  Set current child to 0, call it before 1st time call nextChild 
  
  NonType* nextChild();
  // Get the next child, of current child is the last one, return NULL; 
 
  const char * name() const { return namestring; }
  // Returns the text representation of the tagexp.
  
#ifdef _OSTORE_
  TagExp* tagexp() { return NULL; }
  // Returns the kola equivalent of an object.

  OSObject *result() { return NULL; }
  // Returns the OSObject equivalent of itself.

#endif // _OSTORE_
protected:
  
  TagExp* one;
  // what is the 1st child? 

  int currChild;
};


#endif
