
/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack(mfc) 
//        Xiaoyu Wang(wangxy) 
//  FILE: KOTag.H 
//  PROJ: COKO, KOLA
//  DATE: 9/00		                                                    
//
// SYNOPSIS
//  The class of containable objects.  (Includes
//  integers, strings, as well as pairs and sets of 
//  Objects).  Also should include user defined 
//  typed objects.  This class and its subclasses
//  construct object that are used in KOLA trees.
//  These should be distinguished from
//  OResults which are used in expression trees.
//  KOLA trees are used for optimization.  Expression
//  trees are used to describe the result of a query denoted 
//  by a KOLA tree.
//  
//////////////////////////////////////////////////////////////////////////////

#ifndef KOLA_TAG_HEADER
#define KOLA_TAG_HEADER 

#include "KGlobals.H"
#include "KONameUnion.H"
#include "KONonType.H"
#include "TYTObj.H"

#ifdef _OSTORE_
#include "OSObject.H"
#endif // _OSTORE_

#include "PCState.H"
#include <iostream.h>

class Tag : public NonType
{
public:
  Tag(TagName ten = TAG_INVALID, char* n = NULL);

  Tag(const Tag& other);

  virtual ~Tag ();
  
  virtual Tag* Copy() const = 0;
  // Virtual Copy Constructor.

  NonType* Copy_kTree() const { return Copy(); }

  TagName Identity() const { return myname; }
  // what kind of tag are you?
  // return myname.
        
  virtual char *oql ();
  // Return the OQL string equivalent of this object
  
  virtual char *sql ();
  // Return the SQL string equivalent of this object
  
  virtual Tag *tag();
  // Symbolic simplification of this.  Eventually,
  // this method will replace "object" presently
  // only defined with _OSTORE_ flag set
  
#ifdef _OSTORE_
  
  virtual Tag *tag() = 0;
  // Returns the kola equivalent of an object.
  
  virtual OSObject *result() = 0;
  // Returns the OSObject equivalent of itself.
  
  virtual int equal(Tag* o) { return (this == o); }
  
  virtual int unequal(Tag* o) { return (this != o); }

#endif // _OSTORE_
  
protected:
  TagName  myname;
  // what kind of tag are you?
  
  char * namestring;
  // Text representation for the name of the object.

};
#endif



