/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/*
 * Copyright 1995, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *	Name:
 *	Organization:
 *	Address (postal and/or electronic):
 *
 * To:
 *	Software Librarian
 *	Computer Science Department, Box 1910
 *	Brown University
 *	Providence, RI 02912
 *
 *		or
 *
 *	brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.


//////////////////////////////////////////////////////////////////////////////
//                                                                          
//  NAME: Mitch Cherniack (mfc@cs.brown.edu)					 	    
//  FILE: Kola.l 
//  PROJ: KOLA Parser and Type Generator
//  DATE: 5/95		                                                    
//
//  Lex file specifying tokens manipulated by Yacc for KOLA parser
//  and tree generator
//
//////////////////////////////////////////////////////////////////////////////
*/


%{
#include "Kola.tab.h"
#include "KGlobals.H"
#include <string.h>


int lineno = 1; 

%}

%a 3000

%%
[ \t]+                                                            {}
[\n\f]                                                            { lineno++; }
[-][-][^\n]+                                                      { return COMMENT; }
[+-]?[0-9]+                                                       { Kolalval.sval = strdup(yytext);
                                                                    return INTCON; }
\"[^\"]*[\"]                                                      { Kolalval.sval = strdup(yytext);
                                                                    return STRINGCON; }
[+-]?(([0-9]+)|([0-9]*\.[0-9]+)([eE][-+]?[0-9]+)?)                { Kolalval.sval = strdup(yytext);
                                                                    return (FLOATCON); }
[Oo][Ii][Dd]                                                      { return OID; }

"{"                                                               { return ('{'); }  
"}"                                                               { return ('}'); }    
"{|"                                                              { return DPO; }   
"|}"                                                              { return DPC; }   

[Ss][Tt][Rr]                                                      { return KSTRING; }
[Rr][Ee][Aa][Ll]                                                  { return KFLOAT; }
[Cc][Hh][Aa][Rr]                                                  { return KCHAR;}
"="                                                               { return (PEQUAL); }  
"<"                                                               { return (LANGLE); }  
">"                                                               { return (RANGLE); }
[Ll][Tt]                                                          { return LESS; }
[Gg][Tt]                                                          { return GREATER; }
[Ll][Ee][Qq]                                                      { return LE; } 
[Gg][Ee][Qq]                                                      { return GE; } 
"<="                                                              { return LE; } 
">="                                                              { return GE; } 
"!="                                                              { return NE; } 
","                                                               { return (','); }  
"("                                                               { return ( '(' ); }  
")"                                                               { return ( ')' ); }
"+"                                                               { return (PLUS); }  
"-"                                                               { return (MINUS); }  
"*"                                                               { return (MULT); }  
"/"                                                               { return (DIV); }  
"["                                                               { return ('['); }  
"]"                                                               { return (']'); }  
"&"                                                               { return ('&'); }  
"|"                                                               { return ('|'); }  
"\\/"           					          { return (BOR); }  
"/\\"				                                  { return (BAND); }  
"~"            		                                          { return (NOT); }  
"@"								  { return ('@'); }
"%"								  { return ('%'); }

[Pp][Rr][Ee][Dd]  	      	                                  { return (PPREFIX); }
[Ff][Uu][Nn]			                                  { return (FPREFIX); }
[Oo][Bb][Jj]			                                  { return (OPREFIX); }
[Bb][Oo][Oo][Ll]		                                  { return (BPREFIX); }

"$"								  { return ('$');     }
"!!"								  { return (DOUBLEBANG);}
"<-"								  { return (LARROW);  }
"\\|"								  { return (ALT);     }

[sS][tT][aA][rR]						  { return (STAR);    }
[sS][tT]							  { return (ST);      }
[mM][aA][tT][cC][hH]						  { return (MATCH);   }
[eE][xX][tT][rR][aA][cC][tT]					  { return (FEXTRACT); }

"!"								  { return (FINVOKE); }
"?"								  { return (PINVOKE); }
"o"					                          { return (CIRC); }  
"K"             						  { return (CONST); }  
"C"             						  { return (CURRY); }  
[eE][lL][tT]                                                      { return (ELT); }
[Ee][Ll][Ee]                                                      { return (ELE);   }
[Aa][Pp][Pp]                                                      { return (APP);   }
[Ss][Hh][Ll]                                                      { return (SHIFTL); }
[Ss][Hh][Rr]                                                      { return (SHIFTR); }      
[Aa][Vv][Gg]                                                      { return (FAVG); }
[Mm][Aa][Xx]                                                      { return (FMAX); }      
[Mm][Ii][Nn]                                                      { return (FMIN); }      
[Ss][Uu][Mm]                                                      { return (FSUM); }      
[Cc][Oo][Uu][Nn][Tt]                                              { return (FCOUNT); }      
[Ll][Ee][Xx]                                                      { return (PLEX); }      
[Aa][Dd][Dd]                                                      { return (FADD); }      
[Mm][Ii][Nn][Uu][Ss]                                              { return (FSUB); }  
[Mm][Uu][Ll]                                                      { return (FMUL); }  
[Dd][Ii][Vv]                                                      { return (FDIV); }  
[Nn][Ii][Ll]                                                      { return (NIL); }  
[Ee][Qq][Uu][Aa][Ll]                                              { return (PEQUAL); }  
[Ll][Ee][Ss][Ss]                                                  { return (LESS); }  
[Gg][Rr][Ee][Aa][Tt][Ee][Rr]                                      { return (GREATER); }  
[Ff][Aa][Ll][Ss][Ee]                                              { return (FALSEV); }  
[Ii][Tt][Ee][Rr][Aa][Tt][Ee]                                      { return (ITERATE); }  
[Ii][Tt][Ee][Rr]                                                  { return (ITER); } 
[Pp][Cc][Oo][Nn]                                                  { return (PCON); }   
[Pp][Ff][Oo][Ll][Dd]                                              { return (PFOLD); }  
[Ss][Ee][Tt]                                                      { return (SET); }  
[Ss][Ii][Nn][Gg][Ll][Ee][Tt][Oo][Nn]                              { return (SINGLETON); }
[Ff][Oo][Ll][Dd]                                                  { return (FOLD); }  
[Uu][Nn][Ii]                                                      { return (UNI); }  
[Ii][Nn][Tt]                                                      { return (INT); }
[vV][aA][lL][uU][eE]                                              { return (VALUE);}  
[Ii][Ff]	                                                  { return (IF); }  
[Tt][Hh][Ee][Nn]                                                  { return (FTHEN); }  
[Ee][Ll][Ss][Ee]                                                  { return (ELSE); }  
[Ff][Ii]	                                                  { return (FI); }  
[Dd][Ii][Ff]                                                      { return (DIF); }  
[Jj][Oo][Ii][Nn]                                                  { return (JOIN); }  
[Nn][Jj][Oo][Ii][Nn]                                              { return (NJOIN); }
[Uu][Nn][Nn][Ee][Ss][Tt]                                          { return (UNNEST); }  
[Ff][Ll][Aa][Tt]                                                  { return (FLATTEN); }  
[Cc][Hh][Oo][Oo][Ss][Ee]                                          { return (CHOOSE); }  
[Ee][Mm][Pp][Tt][Yy]                                              { return (EMPTY); }  
[Rr][Ss][Jj][Oo][Ii][Nn]                                          { return (RSJOIN); }  
[Ll][Ss][Jj][Oo][Ii][Nn]                                          { return (LSJOIN); }  
[Ii][Nn][Ss]                                                      { return (INS); }  
[Ss][Oo][Rr][Tt]                                                  { return (SORT); }  
[Aa][Nn][Dd]  	   	      	             	       	    	  { return (BAND); }  
[Oo][Rr]  	       	                     	       	    	  { return (BOR); }  
[Nn][Oo][Tt]  	      	       	        	    	          { return (NOT); }  
[Ee][Xx][Ii][Ss][Tt][Ss]       	        	     	       	  { return (EXISTS); }  
[Ff][Oo][Rr][Aa][Ll][Ll]     	      	     	     	       	  { return (FORALL); }  
[Ii][Dd]                       	            			  { return (ID); }  
[Tt][Ii][Dd]                           	            		  { return (TID); }  
[#][1]                    	                		  { return (ONE); }  
[#][2]                                                            { return (TWO); }  
[Tt][Rr][Uu][Ee]	    	                                  { return (TRUEV); }  
[Mm][Oo][Dd]				                          { return (MOD); }
[Aa][Bb][Ss]			                          	  { return (ABS); }
[Cc][Oo][Nn]           						  { return (CON); } 
[Ee][Xx]            					          { return (EX); }   
[Ff][Aa]	           					  { return (FA); }   

[Cc][Ii][Rr][Cc]					          { return (CIRC); }  
[Tt][Ii][Mm][Ee][Ss]			                          { return (TIMES); }  
[X]                                                               { return (XTIMES); }
\\[Ii][Nn][Vv]      						  { return (FINV); }  
[Ii][Nn][Vv]      						  { return (INV); }  
\\[Uu]                          				  { return (UNION); }   
\\[Ii]                          				  { return (INTER); }   
[Pp][Ll][Uu][Ss]                                                  { return (PLUS); }      
[Oo][Pp][Ll][Uu][Ss]                                              { return (OPLUS); }    
[Ee][Xx][Cc][Ee][Pp][Tt]                                          { return (ODIF); }   

 
[a-zA-Z][a-z_A-Z0-9]*                    			  { Kolalval.sval = strdup(yytext);
                                                                    return (IDENT); }




[(][@%][ ]*[a-zA-Z][a-z_A-Z0-9]*[)]

.                    						                      { return (ERROR); }
%%

int yywrap ()  { return 1;}
