/*
 * Copyright 1994, Brown University, Providence, RI
 *
 * Permission to use and modify this software and its documentation for
 * any purpose other than its incorporation into a commercial product is
 * hereby granted without fee.  Permission to copy and distribute this
 * software and its documentation only for non-commercial use is also
 * granted without fee, provided, however, that the above copyright notice
 * appear in all copies, that both that copyright notice and this permission
 * notice appear in supporting documentation, that the name of Brown
 * University not be used in advertising or publicity pertaining to
 * distribution of the software without specific, written prior permission,
 * and that the person doing the distribution notify Brown University of
 * such distributions outside of his or her organization. Brown University
 * makes no representations about the suitability of this software for
 * any purpose.  It is provided "as is" without express or implied warranty.
 * Brown University requests notification of any modifications to this
 * software or its documentation.
 *
 * Send the following redistribution information:
 *
 *      Name:
 *      Organization:
 *      Address (postal and/or electronic):
 *
 * To:
 *      Software Librarian
 *      Computer Science Department, Box 1910
 *      Brown University
 *      Providence, RI 02912
 *
 *              or
 *
 *      brusd@cs.brown.edu
 *
 * We will acknowledge all electronic notifications.
 */

/* -*-Mode:c++-*- */

//////////////////////////////////////////////////////////////////////////////
//
// NAME: Mitch Cherniack (mfc@cs.brown.edu)					 	 
// FILE: TYTPred.C
// PROJ: KOLA Parser, AQUA Translator
// DATE: 5/95
//
//////////////////////////////////////////////////////////////////////////////

#include "TYTPred.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  TPred
// Parameters:	(TObj* o)							  
//
//  Constructor.  Argument is the domain type of the predicate.
//         								    
//////////////////////////////////////////////////////////////////////////////

TPred::TPred (TObj* o)
{

    one = o;
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  TPred
// Parameters:	(TPred& t)							  
//
//  Copy Constructor.  
//         								    
//////////////////////////////////////////////////////////////////////////////

TPred::TPred (TPred& t)
{

    one = t . Dom () -> Copy ();
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  representation
// Parameters:	()							  
//
//  Return string representation of type.
//  
//    one -> representation () + "*"
//         								    
//////////////////////////////////////////////////////////////////////////////

char* TPred::representation ()
{
    
    char *orr = one -> representation ();
    
    char *res = new char [8 + strlen (orr)];
    sprintf (res, "PRED: %s*", orr);

    delete [] orr;
    
    return res;
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  prolog
// Parameters:	()							  
//
//  Return string prolog of type.
//  
//    one -> prolog () + "*"
//         								    
//////////////////////////////////////////////////////////////////////////////

char* TPred::prolog ()
{
  char *orr;
  if (one == NULL) {
	orr = new char [strlen ("knull") + 1];
	sprintf (orr, "knull");
  } else
    orr = one -> prolog ();
  char *res = new char [strlen (orr) + strlen ("tpred()") + 1];
  sprintf (res, "tpred(%s)", orr);
  delete [] orr;
  return res;    
}

#ifdef _PROLOG_
//////////////////////////////////////////////////////////////////////////////
//
// Function Name:  sp_prolog
// Parameters:	()
//
// Since C/C++ functions only have indirect access to Prolog terms 
// via SP_term_refs, we need to convert kola tree to SP_term_refs.
// Converted kola tree will get passed to prolog runtime system.
//
// See the Sicstus prolog version 3#6 manual for more detailed info on
// the usage of SP_term_refs and other prolog supplied methods.
//
//////////////////////////////////////////////////////////////////////////////

SP_term_ref TPred::sp_prolog ()
{
  SP_term_ref or;
  if (one == NULL) {
	cout << "Calling sp_prolog on TPred with child = NULL: could be a problem" << endl;
	or = NULL;
  } else
	or = one -> sp_prolog ();

  SP_term_ref res = SP_new_term_ref();
  SP_cons_functor(res, SP_atom_from_string("tpred"),1,or);
  return res;
}
#endif // _PROLOG_

TObj* TPred::Copy ()
{

    return (TObj*) new TPred (*this);
    
}


//////////////////////////////////////////////////////////////////////////////
//                                                                          
// Function Name:  ~TPred
// Parameters:	()							  
//
// Destructor.
//        								    
//////////////////////////////////////////////////////////////////////////////

TPred::~TPred ()
{

    delete one;

}









